@@ -25,9 +25,9 @@ package htsjdk.samtools;
 
 import htsjdk.samtools.DuplicateScoringStrategy.ScoringStrategy;
 import htsjdk.samtools.util.CloseableIterator;
-import htsjdk.samtools.util.CoordMath;
 import htsjdk.samtools.util.RuntimeIOException;
 import htsjdk.samtools.util.SequenceUtil;
+import htsjdk.samtools.util.TestUtil;
 
 import java.io.File;
 import java.io.IOException;
@@ -58,7 +58,7 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
     private static final byte[] BASES = {'A', 'C', 'G', 'T'};
     private static final String READ_GROUP_ID = "1";
     private static final String SAMPLE = "FREE_SAMPLE";
-    private final Random random = new Random();
+    private final Random random = new Random(TestUtil.RANDOM_SEED);
 
     private SAMFileHeader header;
     private final Collection<SAMRecord> records;
@@ -193,16 +193,21 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
     }
 
     /** Returns a CloseableIterator over the collection of SAMRecords. */
+    @Override
     public CloseableIterator<SAMRecord> iterator() {
         return new CloseableIterator<SAMRecord>() {
             private final Iterator<SAMRecord> iterator = records.iterator();
 
+            @Override
             public void close() { /** Do nothing. */}
 
+            @Override
             public boolean hasNext() { return this.iterator.hasNext(); }
 
+            @Override
             public SAMRecord next() { return this.iterator.next(); }
 
+            @Override
             public void remove() { this.iterator.remove(); }
         };
     }
@@ -282,7 +287,7 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
                              final boolean recordUnmapped, final String cigar, final String qualityString,
                              final int defaultQuality, final boolean isSecondary) throws SAMException {
         final htsjdk.samtools.SAMRecord rec = createReadNoFlag(name, contig, start, negativeStrand, recordUnmapped, cigar, qualityString, defaultQuality);
-        if (isSecondary) rec.setNotPrimaryAlignmentFlag(true);
+        if (isSecondary) rec.setSecondaryAlignment(true);
         this.records.add(rec);
         return rec;
     }
@@ -295,7 +300,7 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
                              final boolean recordUnmapped, final String cigar, final String qualityString,
                              final int defaultQuality, final boolean isSecondary, final boolean isSupplementary) throws SAMException {
         final htsjdk.samtools.SAMRecord rec = createReadNoFlag(name, contig, start, negativeStrand, recordUnmapped, cigar, qualityString, defaultQuality);
-        if (isSecondary) rec.setNotPrimaryAlignmentFlag(true);
+        if (isSecondary) rec.setSecondaryAlignment(true);
         if (isSupplementary) rec.setSupplementaryAlignmentFlag(true);
         this.records.add(rec);
         return rec;
@@ -358,13 +363,8 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
         end1.setMappingQuality(255);
         end1.setReadPairedFlag(true);
         end1.setProperPairFlag(true);
-        end1.setMateReferenceIndex(contig);
-        end1.setAttribute(SAMTag.MC.name(), readLength + "M");
-        end1.setMateAlignmentStart(start2);
-        end1.setMateNegativeStrandFlag(true);
         end1.setFirstOfPairFlag(end1IsFirstOfPair);
         end1.setSecondOfPairFlag(!end1IsFirstOfPair);
-        end1.setInferredInsertSize((int) CoordMath.getLength(start1, CoordMath.getEnd(start2, this.readLength)));
         end1.setAttribute(SAMTag.RG.name(), READ_GROUP_ID);
         if (programRecord != null) {
             end1.setAttribute(SAMTag.PG.name(), programRecord.getProgramGroupId());
@@ -383,13 +383,8 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
         end2.setMappingQuality(255);
         end2.setReadPairedFlag(true);
         end2.setProperPairFlag(true);
-        end2.setMateReferenceIndex(contig);
-        end2.setAttribute(SAMTag.MC.name(), readLength + "M");
-        end2.setMateAlignmentStart(start1);
-        end2.setMateNegativeStrandFlag(false);
         end2.setFirstOfPairFlag(!end1IsFirstOfPair);
         end2.setSecondOfPairFlag(end1IsFirstOfPair);
-        end2.setInferredInsertSize(end1.getInferredInsertSize());
         end2.setAttribute(SAMTag.RG.name(), READ_GROUP_ID);
         if (programRecord != null) {
             end2.setAttribute(SAMTag.PG.name(), programRecord.getProgramGroupId());
@@ -399,6 +394,9 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
         }
         fillInBasesAndQualities(end2);
 
+        // set mate info
+        SamPairUtil.setMateInfo(end1, end2, true);
+
         this.records.add(end1);
         this.records.add(end2);
     }
@@ -436,11 +434,11 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
         end2.setReadPairedFlag(true);
         end2.setSecondOfPairFlag(true);
 
-        if (record1NonPrimary) end1.setNotPrimaryAlignmentFlag(true);
-        if (record2NonPrimary) end2.setNotPrimaryAlignmentFlag(true);
+        if (record1NonPrimary) end1.setSecondaryAlignment(true);
+        if (record2NonPrimary) end2.setSecondaryAlignment(true);
 
-        if (record1NonPrimary) end1.setNotPrimaryAlignmentFlag(true);
-        if (record2NonPrimary) end2.setNotPrimaryAlignmentFlag(true);
+        if (record1NonPrimary) end1.setSecondaryAlignment(true);
+        if (record2NonPrimary) end2.setSecondaryAlignment(true);
 
         // set mate info
         SamPairUtil.setMateInfo(end1, end2, true);
@@ -487,7 +485,7 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
             end1.setAttribute(SAMTag.PG.name(), programRecord.getProgramGroupId());
         }
         if (this.unmappedHasBasesAndQualities) {
-        fillInBasesAndQualities(end1);
+            fillInBasesAndQualities(end1);
         }
 
         end2.setReadName(name);
@@ -503,7 +501,7 @@ public class SAMRecordSetBuilder implements Iterable<SAMRecord> {
             end2.setAttribute(SAMTag.PG.name(), programRecord.getProgramGroupId());
         }
         if (this.unmappedHasBasesAndQualities) {
-        fillInBasesAndQualities(end2);
+            fillInBasesAndQualities(end2);
         }
 
         this.records.add(end1);
