@@ -43,14 +43,17 @@ import java.util.Map;
 import java.util.TreeMap;
 import java.util.regex.Pattern;
 
-
 /**
  * Utilty methods.
  */
 public final class SAMUtils {
-    /** regex for semicolon, used in {@link SAMUtils#getOtherCanonicalAlignments(SAMRecord)} */
+    /**
+     * regex for semicolon, used in {@link SAMUtils#getOtherCanonicalAlignments(SAMRecord)}
+     */
     private static final Pattern SEMICOLON_PAT = Pattern.compile("[;]");
-    /** regex for comma, used in {@link SAMUtils#getOtherCanonicalAlignments(SAMRecord)} */
+    /**
+     * regex for comma, used in {@link SAMUtils#getOtherCanonicalAlignments(SAMRecord)}
+     */
     private static final Pattern COMMA_PAT = Pattern.compile("[,]");
 
     // Representation of bases, one for when in low-order nybble, one for when in high-order nybble.
@@ -87,32 +90,31 @@ public final class SAMUtils {
     private static final byte COMPRESSED_K_HIGH = (byte) (COMPRESSED_K_LOW << 4);
     private static final byte COMPRESSED_D_HIGH = (byte) (COMPRESSED_D_LOW << 4);
     private static final byte COMPRESSED_B_HIGH = (byte) (COMPRESSED_B_LOW << 4);
-    
-    private static final byte [] COMPRESSED_LOOKUP_TABLE = 
-            new byte[]{
-                '=',
-                'A',
-                'C',
-                'M',
-                'G',
-                'R',
-                'S',
-                'V',
-                'T',
-                'W',
-                'Y',
-                'H',
-                'K',
-                'D',
-                'B',
-                'N'
-            };
-    
+
+    private static final byte[] COMPRESSED_LOOKUP_TABLE = {
+            '=',
+            'A',
+            'C',
+            'M',
+            'G',
+            'R',
+            'S',
+            'V',
+            'T',
+            'W',
+            'Y',
+            'H',
+            'K',
+            'D',
+            'B',
+            'N'
+    };
+
     public static final int MAX_PHRED_SCORE = 93;
 
     /**
-     * Convert from a byte array containing =AaCcGgTtNn represented as ASCII, to a byte array half as long,
-     * with =, A, C, G, T converted to 0, 1, 2, 4, 8, 15.
+     * Convert from a byte array containing =AaCcGgTtNnMmRrSsVvWwYyHhKkDdBb represented as ASCII, to a byte array half as long,
+     * with for example, =, A, C, G, T converted to 0, 1, 2, 4, 8, 15.
      *
      * @param readBases Bases as ASCII bytes.
      * @return New byte array with bases represented as nybbles, in BAM binary format.
@@ -126,17 +128,17 @@ public final class SAMUtils {
         }
         // Last nybble
         if (i == readBases.length) {
-            compressedBases[i / 2] = charToCompressedBaseHigh((char) readBases[i - 1]);
+            compressedBases[i / 2] = charToCompressedBaseHigh(readBases[i - 1]);
         }
         return compressedBases;
     }
 
     /**
-     * Convert from a byte array with basese stored in nybbles, with =, A, C, G, T represented as 0, 1, 2, 4, 8, 15,
+     * Convert from a byte array with bases stored in nybbles, with for example,=, A, C, G, T, N represented as 0, 1, 2, 4, 8, 15,
      * to a a byte array containing =AaCcGgTtNn represented as ASCII.
      *
-     * @param length Number of bases (not bytes) to convert.
-     * @param compressedBases Bases represented as nybbles, in BAM binary format.
+     * @param length           Number of bases (not bytes) to convert.
+     * @param compressedBases  Bases represented as nybbles, in BAM binary format.
      * @param compressedOffset Byte offset in compressedBases to start.
      * @return New byte array with bases as ASCII bytes.
      */
@@ -158,10 +160,11 @@ public final class SAMUtils {
     /**
      * Convert from ASCII byte to BAM nybble representation of a base in low-order nybble.
      *
-     * @param base One of =AaCcGgTtNn.
+     * @param base One of =AaCcGgTtNnMmRrSsVvWwYyHhKkDdBb.
      * @return Low-order nybble-encoded equivalent.
+     * @throws IllegalArgumentException if the base is not one of =AaCcGgTtNnMmRrSsVvWwYyHhKkDdBb.
      */
-    private static byte charToCompressedBaseLow(final int base) {
+    private static byte charToCompressedBaseLow(final byte base) {
         switch (base) {
             case '=':
                 return COMPRESSED_EQUAL_LOW;
@@ -214,17 +217,18 @@ public final class SAMUtils {
             case 'b':
                 return COMPRESSED_B_LOW;
             default:
-                throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
+                throw new IllegalArgumentException("Bad base passed to charToCompressedBaseLow: " + Character.toString((char) base) + "(" + base + ")");
         }
     }
 
     /**
      * Convert from ASCII byte to BAM nybble representation of a base in high-order nybble.
      *
-     * @param base One of =AaCcGgTtNn.
+     * @param base One of =AaCcGgTtNnMmRrSsVvWwYyHhKkDdBb.
      * @return High-order nybble-encoded equivalent.
+     * @throws IllegalArgumentException if the base is not one of =AaCcGgTtNnMmRrSsVvWwYyHhKkDdBb.
      */
-    private static byte charToCompressedBaseHigh(final int base) {
+    private static byte charToCompressedBaseHigh(final byte base) {
         switch (base) {
             case '=':
                 return COMPRESSED_EQUAL_HIGH;
@@ -277,20 +281,22 @@ public final class SAMUtils {
             case 'b':
                 return COMPRESSED_B_HIGH;
             default:
-                throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
+                throw new IllegalArgumentException("Bad base passed to charToCompressedBaseHigh: " + Character.toString((char) base) + "(" + base + ")");
         }
     }
-    
+
     /**
      * Returns the byte corresponding to a certain nybble
+     *
      * @param base One of COMPRESSED_*_LOW, a low-order nybble encoded base.
-     * @return ASCII base, one of ACGTN=.
+     * @return ASCII base, one of =ACGTNMRSVWYHKDB.
+     * @throws IllegalArgumentException if the base is not one of =ACGTNMRSVWYHKDB.
      */
-    private static byte compressedBaseToByte(byte base){
-        try{
+    private static byte compressedBaseToByte(byte base) {
+        try {
             return COMPRESSED_LOOKUP_TABLE[base];
-        }catch(IndexOutOfBoundsException e){
-            throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
+        } catch (IndexOutOfBoundsException e) {
+            throw new IllegalArgumentException("Bad base passed to charToCompressedBase: " + Character.toString((char) base) + "(" + base + ")");
         }
     }
 
@@ -301,7 +307,7 @@ public final class SAMUtils {
      * @return ASCII base, one of ACGTN=.
      */
     private static byte compressedBaseToByteLow(final int base) {
-        return compressedBaseToByte((byte)(base & 0xf));
+        return compressedBaseToByte((byte) (base & 0xf));
     }
 
     /**
@@ -311,13 +317,13 @@ public final class SAMUtils {
      * @return ASCII base, one of ACGTN=.
      */
     private static byte compressedBaseToByteHigh(final int base) {
-        return compressedBaseToByte((byte)((base >> 4) & 0xf));
+        return compressedBaseToByte((byte) ((base >> 4) & 0xf));
     }
 
     /**
      * Convert bases in place into canonical form, upper case, and with no-call represented as N.
      *
-     * @param bases
+     * @param bases byte array of bases to "normalize", in place.
      */
     static void normalizeBases(final byte[] bases) {
         for (int i = 0; i < bases.length; ++i) {
@@ -431,11 +437,11 @@ public final class SAMUtils {
     /**
      * Handle a list of validation errors according to the validation stringency.
      *
-     * @param validationErrors List of errors to report, or null if there are no errors.
-     * @param samRecordIndex Record number of the SAMRecord corresponding to the validation errors, or -1 if
-     * the record number is not known.
+     * @param validationErrors     List of errors to report, or null if there are no errors.
+     * @param samRecordIndex       Record number of the SAMRecord corresponding to the validation errors, or -1 if
+     *                             the record number is not known.
      * @param validationStringency If STRICT, throw a SAMFormatException.  If LENIENT, print the validation
-     * errors to stderr.  If SILENT, do nothing.
+     *                             errors to stderr.  If SILENT, do nothing.
      */
     public static void processValidationErrors(final List<SAMValidationError> validationErrors,
                                                final long samRecordIndex,
@@ -461,11 +467,10 @@ public final class SAMUtils {
         } else if (validationStringency == ValidationStringency.LENIENT) {
             System.err.println("Ignoring SAM validation error: " + validationError);
         }
-
     }
 
     private static final SAMHeaderRecordComparator<SAMReadGroupRecord> HEADER_RECORD_COMPARATOR =
-            new SAMHeaderRecordComparator<SAMReadGroupRecord>(
+            new SAMHeaderRecordComparator<>(
                     SAMReadGroupRecord.PLATFORM_UNIT_TAG,
                     SAMReadGroupRecord.LIBRARY_TAG,
                     SAMReadGroupRecord.DATE_RUN_PRODUCED_TAG,
@@ -473,7 +478,8 @@ public final class SAMUtils {
                     SAMReadGroupRecord.SEQUENCING_CENTER_TAG,
                     SAMReadGroupRecord.PLATFORM_TAG,
                     SAMReadGroupRecord.DESCRIPTION_TAG,
-                    SAMReadGroupRecord.READ_GROUP_ID_TAG    // We don't actually want to compare with ID but it's suitable
+                    SAMReadGroupRecord.READ_GROUP_ID_TAG
+                    // We don't actually want to compare with ID but it's suitable
                     // "just in case" since it's the only one that's actually required
             );
 
@@ -494,11 +500,11 @@ public final class SAMUtils {
 
         // Sort the read group records by their first
         final SamReader reader = SamReaderFactory.makeDefault().referenceSequence(referenceFasta).open(input);
-        final List<SAMReadGroupRecord> sortedRecords = new ArrayList<SAMReadGroupRecord>(reader.getFileHeader().getReadGroups());
+        final List<SAMReadGroupRecord> sortedRecords = new ArrayList<>(reader.getFileHeader().getReadGroups());
         Collections.sort(sortedRecords, HEADER_RECORD_COMPARATOR);
 
         for (final SAMReadGroupRecord rgRecord : sortedRecords) {
-            final TreeMap<String, String> sortedAttributes = new TreeMap<String, String>();
+            final TreeMap<String, String> sortedAttributes = new TreeMap<>();
             for (final Map.Entry<String, String> attributeEntry : rgRecord.getAttributes()) {
                 sortedAttributes.put(attributeEntry.getKey(), attributeEntry.getValue());
             }
@@ -536,7 +542,7 @@ public final class SAMUtils {
 
         final List<SAMProgramRecord> pgs = header.getProgramRecords();
         if (!pgs.isEmpty()) {
-            final List<String> referencedIds = new ArrayList<String>();
+            final List<String> referencedIds = new ArrayList<>();
             for (final SAMProgramRecord pg : pgs) {
                 if (pg.getPreviousProgramGroupId() != null) {
                     referencedIds.add(pg.getPreviousProgramGroupId());
@@ -557,7 +563,7 @@ public final class SAMUtils {
 
     /**
      * Strip mapping information from a SAMRecord.
-     *
+     * <p>
      * WARNING: by clearing the secondary and supplementary flags,
      * this may have the affect of producing multiple distinct records with the
      * same read name and flags, which may lead to invalid SAM/BAM output.
@@ -565,7 +571,7 @@ public final class SAMUtils {
      */
     public static void makeReadUnmapped(final SAMRecord rec) {
         if (rec.getReadNegativeStrandFlag()) {
-            SAMRecordUtil.reverseComplement(rec);
+            rec.reverseComplement(true);
             rec.setReadNegativeStrandFlag(false);
         }
         rec.setDuplicateReadFlag(false);
@@ -574,7 +580,7 @@ public final class SAMUtils {
         rec.setCigarString(SAMRecord.NO_ALIGNMENT_CIGAR);
         rec.setMappingQuality(SAMRecord.NO_MAPPING_QUALITY);
         rec.setInferredInsertSize(0);
-        rec.setNotPrimaryAlignmentFlag(false);
+        rec.setSecondaryAlignment(false);
         rec.setSupplementaryAlignmentFlag(false);
         rec.setProperPairFlag(false);
         rec.setReadUnmappedFlag(true);
@@ -619,13 +625,13 @@ public final class SAMUtils {
     /**
      * Tests if the provided record is mapped entirely beyond the end of the reference (i.e., the alignment start is greater than the
      * length of the sequence to which the record is mapped).
+     *
      * @param record must not have a null SamFileHeader
      */
     public static boolean recordMapsEntirelyBeyondEndOfReference(final SAMRecord record) {
         if (record.getHeader() == null) {
             throw new SAMException("A non-null SAMHeader is required to resolve the mapping position: " + record.getReadName());
-        }
-        else {
+        } else {
             return record.getHeader().getSequence(record.getReferenceIndex()).getSequenceLength() < record.getAlignmentStart();
         }
     }
@@ -643,7 +649,6 @@ public final class SAMUtils {
         else return mapq1 - mapq2;
     }
 
-
     /**
      * Hokey algorithm for combining two MAPQs into values that are comparable, being cognizant of the fact
      * that in MAPQ world, 1 > 255 > 0. In this algorithm, 255 is treated as if it were 0.01, so that
@@ -652,11 +657,17 @@ public final class SAMUtils {
      * invocations of this method.
      */
     public static int combineMapqs(int m1, int m2) {
-        if (m1 == 255) m1 = 1;
-        else m1 *= 100;
+        if (m1 == 255) {
+            m1 = 1;
+        } else {
+            m1 *= 100;
+        }
 
-        if (m2 == 255) m2 = 1;
-        else m2 *= 100;
+        if (m2 == 255) {
+            m2 = 1;
+        } else {
+            m2 *= 100;
+        }
 
         return m1 + m2;
 
@@ -679,15 +690,15 @@ public final class SAMUtils {
      * reference sequence. Note that clipped portions, and inserted and deleted bases (vs. the reference)
      * are not represented in the alignment blocks.
      *
-     * @param cigar The cigar containing the alignment information
+     * @param cigar          The cigar containing the alignment information
      * @param alignmentStart The start (1-based) of the alignment
-     * @param cigarTypeName The type of cigar passed - for error logging.
+     * @param cigarTypeName  The type of cigar passed - for error logging.
      * @return List of alignment blocks
      */
     public static List<AlignmentBlock> getAlignmentBlocks(final Cigar cigar, final int alignmentStart, final String cigarTypeName) {
         if (cigar == null) return Collections.emptyList();
 
-        final List<AlignmentBlock> alignmentBlocks = new ArrayList<AlignmentBlock>();
+        final List<AlignmentBlock> alignmentBlocks = new ArrayList<>();
         int readBase = 1;
         int refBase = alignmentStart;
 
@@ -718,7 +729,7 @@ public final class SAMUtils {
                     refBase += length;
                     break;
                 default:
-                    throw new IllegalStateException("Case statement didn't deal with " + cigarTypeName + " op: " + e.getOperator());
+                    throw new IllegalStateException("Case statement didn't deal with " + cigarTypeName + " op: " + e.getOperator() + "in CIGAR: " + cigar);
             }
         }
         return Collections.unmodifiableList(alignmentBlocks);
@@ -726,7 +737,7 @@ public final class SAMUtils {
 
     /**
      * @param alignmentStart The start (1-based) of the alignment
-     * @param cigar The cigar containing the alignment information
+     * @param cigar          The cigar containing the alignment information
      * @return the alignment start (1-based, inclusive) adjusted for clipped bases.  For example if the read
      * has an alignment start of 100 but the first 4 bases were clipped (hard or soft clipped)
      * then this method will return 96.
@@ -750,7 +761,7 @@ public final class SAMUtils {
 
     /**
      * @param alignmentEnd The end (1-based) of the alignment
-     * @param cigar The cigar containing the alignment information
+     * @param cigar        The cigar containing the alignment information
      * @return the alignment end (1-based, inclusive) adjusted for clipped bases.  For example if the read
      * has an alignment end of 100 but the last 7 bases were clipped (hard or soft clipped)
      * then this method will return 107.
@@ -788,7 +799,7 @@ public final class SAMUtils {
     /**
      * Returns the Mate Cigar or null if there is none.
      *
-     * @param rec the SAM record
+     * @param rec            the SAM record
      * @param withValidation true if we are to validate the mate cigar before returning, false otherwise.
      * @return Cigar object for the read's mate, or null if there is none.
      */
@@ -832,11 +843,11 @@ public final class SAMUtils {
      */
     public static int getMateAlignmentEnd(final SAMRecord rec) {
         if (rec.getMateUnmappedFlag()) {
-            throw new RuntimeException("getMateAlignmentEnd called on an unmapped mate.");
+            throw new RuntimeException("getMateAlignmentEnd called on an unmapped mate: " + rec);
         }
         final Cigar mateCigar = SAMUtils.getMateCigar(rec);
         if (mateCigar == null) {
-            throw new SAMException("Mate CIGAR (Tag MC) not found.");
+            throw new SAMException("Mate CIGAR (Tag MC) not found:" + rec);
         }
         return CoordMath.getEnd(rec.getMateAlignmentStart(), mateCigar.getReferenceLength());
     }
@@ -851,15 +862,14 @@ public final class SAMUtils {
      */
     public static int getMateUnclippedStart(final SAMRecord rec) {
         if (rec.getMateUnmappedFlag())
-            throw new RuntimeException("getMateUnclippedStart called on an unmapped mate.");
+            throw new RuntimeException("getMateUnclippedStart called on an unmapped mate: " + rec);
         final Cigar mateCigar = getMateCigar(rec);
         if (mateCigar == null) {
-            throw new SAMException("Mate CIGAR (Tag MC) not found.");
+            throw new SAMException("Mate CIGAR (Tag MC) not found: " + rec);
         }
         return SAMUtils.getUnclippedStart(rec.getMateAlignmentStart(), mateCigar);
     }
 
-
     /**
      * @param rec the SAM record
      * @return the mate alignment end (1-based, inclusive) adjusted for clipped bases.  For example if the mate
@@ -870,20 +880,20 @@ public final class SAMUtils {
      */
     public static int getMateUnclippedEnd(final SAMRecord rec) {
         if (rec.getMateUnmappedFlag()) {
-            throw new RuntimeException("getMateUnclippedEnd called on an unmapped mate.");
+            throw new RuntimeException("getMateUnclippedEnd called on an unmapped mate: " + rec);
         }
         final Cigar mateCigar = SAMUtils.getMateCigar(rec);
         if (mateCigar == null) {
-            throw new SAMException("Mate CIGAR (Tag MC) not found.");
+            throw new SAMException("Mate CIGAR (Tag MC) not found: " + rec);
         }
         return SAMUtils.getUnclippedEnd(getMateAlignmentEnd(rec), mateCigar);
     }
 
     /**
      * @param rec the SAM record
-     * Returns blocks of the mate sequence that have been aligned directly to the
-     * reference sequence. Note that clipped portions of the mate and inserted and
-     * deleted bases (vs. the reference) are not represented in the alignment blocks.
+     *            Returns blocks of the mate sequence that have been aligned directly to the
+     *            reference sequence. Note that clipped portions of the mate and inserted and
+     *            deleted bases (vs. the reference) are not represented in the alignment blocks.
      */
     public static List<AlignmentBlock> getMateAlignmentBlocks(final SAMRecord rec) {
         return getAlignmentBlocks(getMateCigar(rec), rec.getMateAlignmentStart(), "mate cigar");
@@ -893,12 +903,12 @@ public final class SAMUtils {
      * Run all validations of the mate's CIGAR.  These include validation that the CIGAR makes sense independent of
      * placement, plus validation that CIGAR + placement yields all bases with M operator within the range of the reference.
      *
-     * @param rec the SAM record
-     * @param cigar The cigar containing the alignment information
-     * @param referenceIndex The reference index
+     * @param rec             the SAM record
+     * @param cigar           The cigar containing the alignment information
+     * @param referenceIndex  The reference index
      * @param alignmentBlocks The alignment blocks (parsed from the cigar)
-     * @param recordNumber For error reporting.  -1 if not known.
-     * @param cigarTypeName For error reporting.  "Read CIGAR" or "Mate Cigar"
+     * @param recordNumber    For error reporting.  -1 if not known.
+     * @param cigarTypeName   For error reporting.  "Read CIGAR" or "Mate Cigar"
      * @return List of errors, or null if no errors.
      */
 
@@ -913,16 +923,15 @@ public final class SAMUtils {
         if (referenceIndex != SAMRecord.NO_ALIGNMENT_REFERENCE_INDEX) {
             SAMFileHeader samHeader = rec.getHeader();
             if (null == samHeader) {
-                if (ret == null) ret = new ArrayList<SAMValidationError>();
+                if (ret == null) ret = new ArrayList<>();
                 ret.add(new SAMValidationError(SAMValidationError.Type.MISSING_HEADER,
                         cigarTypeName + " A non-null SAMHeader is required to validate cigar elements for: ", rec.getReadName(), recordNumber));
-            }
-            else {
+            } else {
                 final SAMSequenceRecord sequence = samHeader.getSequence(referenceIndex);
                 final int referenceSequenceLength = sequence.getSequenceLength();
                 for (final AlignmentBlock alignmentBlock : alignmentBlocks) {
                     if (alignmentBlock.getReferenceStart() + alignmentBlock.getLength() - 1 > referenceSequenceLength) {
-                        if (ret == null) ret = new ArrayList<SAMValidationError>();
+                        if (ret == null) ret = new ArrayList<>();
                         ret.add(new SAMValidationError(SAMValidationError.Type.CIGAR_MAPS_OFF_REFERENCE,
                                 cigarTypeName + " M operator maps off end of reference", rec.getReadName(), recordNumber));
                         break;
@@ -937,7 +946,7 @@ public final class SAMUtils {
      * Run all validations of the mate's CIGAR.  These include validation that the CIGAR makes sense independent of
      * placement, plus validation that CIGAR + placement yields all bases with M operator within the range of the reference.
      *
-     * @param rec the SAM record
+     * @param rec          the SAM record
      * @param recordNumber For error reporting.  -1 if not known.
      * @return List of errors, or null if no errors.
      */
@@ -951,7 +960,7 @@ public final class SAMUtils {
                 }
             } else {
                 if (getMateCigarString(rec) != null) {
-                    ret = new ArrayList<SAMValidationError>();
+                    ret = new ArrayList<>();
                     if (!rec.getReadPairedFlag()) {
                         // If the read is not paired, and the Mate Cigar String (MC Attribute) exists, that is a validation error
                         ret.add(new SAMValidationError(SAMValidationError.Type.MATE_CIGAR_STRING_INVALID_PRESENCE,
@@ -981,11 +990,11 @@ public final class SAMUtils {
     }
 
     /**
-     * Returns a string that is the the read group ID and read name separated by a colon.  This is meant to cannonically
+     * Returns a string that is the the read group ID and read name separated by a colon.  This is meant to canonically
      * identify a given record within a set of records.
      *
-     * @param record
-     * @return
+     * @param record SAMRecord for which "canonical" read name is requested
+     * @return The record's readgroup-id (if non-null) and the read name, separated by a colon, ':'
      */
     public static String getCanonicalRecordName(final SAMRecord record) {
         String name = record.getStringAttribute(ReservedTagConstants.READ_GROUP_ID);
@@ -999,7 +1008,7 @@ public final class SAMUtils {
      * or the given record's start position is greater than its mate's start position, zero is automatically returned.
      * NB: This method assumes that the record's mate is not contained within the given record's alignment.
      *
-     * @param rec
+     * @param rec SAMRecord that needs clipping due to overlapping pairs.
      * @return the number of bases at the end of the read that need to be clipped such that there would be no overlapping bases with its mate.
      * Read bases include only those from insertion, match, or mismatch Cigar operators.
      */
@@ -1010,7 +1019,8 @@ public final class SAMUtils {
 
         // Only clip records that are left-most in genomic order and overlapping.
         if (rec.getMateAlignmentStart() < rec.getAlignmentStart()) return 0; // right-most, so ignore.
-        else if (rec.getMateAlignmentStart() == rec.getAlignmentStart() && rec.getFirstOfPairFlag()) return 0; // same start, so pick the first end
+        else if (rec.getMateAlignmentStart() == rec.getAlignmentStart() && rec.getFirstOfPairFlag())
+            return 0; // same start, so pick the first end
 
         // Find the number of read bases after the given mate's alignment start.
         int numBasesToClip = 0;
@@ -1023,12 +1033,11 @@ public final class SAMUtils {
             if (refStartPos <= refPos + refBasesLength - 1) { // add to clipped bases
                 if (operator == CigarOperator.MATCH_OR_MISMATCH) { // M
                     if (refStartPos < refPos) numBasesToClip += refBasesLength; // use all of the bases
-                    else numBasesToClip += (refPos + refBasesLength) - refStartPos;  // since the mate's alignment start can be in the middle of a cigar element
-                }
-                else if (operator == CigarOperator.SOFT_CLIP || operator == CigarOperator.HARD_CLIP || operator == CigarOperator.PADDING || operator == CigarOperator.SKIPPED_REGION) {
+                    else
+                        numBasesToClip += (refPos + refBasesLength) - refStartPos;  // since the mate's alignment start can be in the middle of a cigar element
+                } else if (operator == CigarOperator.SOFT_CLIP || operator == CigarOperator.HARD_CLIP || operator == CigarOperator.PADDING || operator == CigarOperator.SKIPPED_REGION) {
                     // ignore
-                }
-                else { // ID
+                } else { // ID
                     numBasesToClip += operator.consumesReadBases() ? el.getLength() : 0; // clip all the bases in the read from this operator
                 }
             }
@@ -1041,14 +1050,14 @@ public final class SAMUtils {
     }
 
     /**
-     * Returns a (possibly new) record that has been clipped if isa  mapped paired and has overlapping bases with its mate.
+     * Returns a (possibly new) record that has been clipped if input is a mapped paired and has overlapping bases with its mate.
      * See {@link #getNumOverlappingAlignedBasesToClip(SAMRecord)} for how the number of overlapping bases is computed.
      * NB: this does not properly consider a cigar like: 100M20S10H.
      * NB: This method assumes that the record's mate is not contained within the given record's alignment.
      *
-     * @param record the record from which to clip bases.
+     * @param record        the record from which to clip bases.
      * @param noSideEffects if true a modified clone of the original record is returned, otherwise we modify the record directly.
-     * @return
+     * @return a (possibly new) record that has been clipped
      */
     public static SAMRecord clipOverlappingAlignedBases(final SAMRecord record, final boolean noSideEffects) {
         return clipOverlappingAlignedBases(record, getNumOverlappingAlignedBasesToClip(record), noSideEffects);
@@ -1060,18 +1069,20 @@ public final class SAMUtils {
      * NB: this does not properly consider a cigar like: 100M20S10H.
      * NB: This method assumes that the record's mate is not contained within the given record's alignment.
      *
-     * @param record the record from which to clip bases.
+     * @param record                    the record from which to clip bases.
      * @param numOverlappingBasesToClip the number of bases to clip at the end of the read.
-     * @param noSideEffects if true a modified clone of the original record is returned, otherwise we modify the record directly.
-     * @return
+     * @param noSideEffects             if true a modified clone of the original record is returned, otherwise we modify the record directly.
+     * @return Returns a (possibly new) SAMRecord with the given number of bases soft-clipped
      */
     public static SAMRecord clipOverlappingAlignedBases(final SAMRecord record, final int numOverlappingBasesToClip, final boolean noSideEffects) {
         // NB: ignores how to handle supplemental records when present for both ends by just using the mate information in the record.
 
-        if (numOverlappingBasesToClip <= 0 || record.getReadUnmappedFlag() || record.getMateUnmappedFlag()) return record;
+        if (numOverlappingBasesToClip <= 0 || record.getReadUnmappedFlag() || record.getMateUnmappedFlag()) {
+            return record;
+        }
 
         try {
-            final SAMRecord rec = noSideEffects ? ((SAMRecord)record.clone()) : record;
+            final SAMRecord rec = noSideEffects ? ((SAMRecord) record.clone()) : record;
 
             // watch out for when the second read overlaps all of the first read
             if (rec.getMateAlignmentStart() <= rec.getAlignmentStart()) { // make it unmapped
@@ -1082,7 +1093,7 @@ public final class SAMUtils {
             // 1-based index of first base in read to clip.
             int clipFrom = rec.getReadLength() - numOverlappingBasesToClip + 1;
             // we have to check if the last cigar element is soft-clipping, so we can subtract that from clipFrom
-            final CigarElement cigarElement = rec.getCigar().getCigarElement(rec.getCigarLength()-1);
+            final CigarElement cigarElement = rec.getCigar().getCigarElement(rec.getCigarLength() - 1);
             if (CigarOperator.SOFT_CLIP == cigarElement.getOperator()) clipFrom -= cigarElement.getLength();
             // FIXME: does not properly consider a cigar like: 100M20S10H
 
@@ -1108,100 +1119,102 @@ public final class SAMUtils {
      * Extract a List of 'other canonical alignments' from a SAM record. Those alignments are stored as a string in the 'SA' tag as defined
      * in the SAM specification.
      * The name, sequence and qualities, mate data are copied from the original record.
+     *
      * @param record must be non null and must have a non-null associated header.
      * @return a list of 'other canonical alignments' SAMRecords. The list is empty if the 'SA' attribute is missing.
      */
     public static List<SAMRecord> getOtherCanonicalAlignments(final SAMRecord record) {
-        if( record == null ) throw new IllegalArgumentException("record is null");
-        if( record.getHeader() == null ) throw new IllegalArgumentException("record.getHeader() is null");
+        if (record == null) throw new IllegalArgumentException("record is null");
+        if (record.getHeader() == null) throw new IllegalArgumentException("record.getHeader() is null");
         /* extract value of SA tag */
-        final Object saValue = record.getAttribute( SAMTagUtil.getSingleton().SA );
-        if( saValue == null ) return Collections.emptyList();
-        if( ! (saValue instanceof String) ) throw new SAMException(
-                "Expected a String for attribute 'SA' but got " + saValue.getClass() );
+        final Object saValue = record.getAttribute(SAMTagUtil.getSingleton().SA);
+        if (saValue == null) return Collections.emptyList();
+        if (!(saValue instanceof String)) throw new SAMException(
+                "Expected a String for attribute 'SA' but got " + saValue.getClass() + ". Record: " + record);
 
         final SAMRecordFactory samReaderFactory = new DefaultSAMRecordFactory();
 
         /* the spec says: "Other canonical alignments in a chimeric alignment, formatted as a
          * semicolon-delimited list: (rname,pos,strand,CIGAR,mapQ,NM;)+.
          * Each element in the list represents a part of the chimeric alignment.
-         * Conventionally, at a supplementary line, the  1rst element points to the primary line.
+         * Conventionally, at a supplementary line, the 1st element points to the primary line.
          */
 
         /* break string using semicolon */
-        final String semiColonStrs[] = SEMICOLON_PAT.split((String)saValue);
+        final String semiColonStrs[] = SEMICOLON_PAT.split((String) saValue);
 
         /* the result list */
-        final List<SAMRecord> alignments = new ArrayList<>( semiColonStrs.length );
+        final List<SAMRecord> alignments = new ArrayList<>(semiColonStrs.length);
 
         /* base SAM flag */
-        int record_flag = record.getFlags() ;
+        int record_flag = record.getFlags();
         record_flag &= ~SAMFlag.PROPER_PAIR.flag;
         record_flag &= ~SAMFlag.SUPPLEMENTARY_ALIGNMENT.flag;
         record_flag &= ~SAMFlag.READ_REVERSE_STRAND.flag;
 
-
-        for(int i=0; i< semiColonStrs.length;++i  ) {
+        for (int i = 0; i < semiColonStrs.length; ++i) {
             final String semiColonStr = semiColonStrs[i];
             /* ignore empty string */
-            if( semiColonStr.isEmpty() ) continue;
+            if (semiColonStr.isEmpty()) continue;
 
             /* break string using comma */
             final String commaStrs[] = COMMA_PAT.split(semiColonStr);
-            if( commaStrs.length != 6 )  throw new SAMException("Bad 'SA' attribute in " + semiColonStr);
+            if (commaStrs.length != 6)
+                throw new SAMException("Bad 'SA' attribute in " + semiColonStr + ". Record: " + record);
 
             /* create the new record */
-            final SAMRecord otherRec = samReaderFactory.createSAMRecord( record.getHeader() );
+            final SAMRecord otherRec = samReaderFactory.createSAMRecord(record.getHeader());
 
             /* copy fields from the original record */
-            otherRec.setReadName( record.getReadName() );
-            otherRec.setReadBases( record.getReadBases() );
-            otherRec.setBaseQualities( record.getBaseQualities() );
-            if( record.getReadPairedFlag() && !record.getMateUnmappedFlag()) {
-                otherRec.setMateReferenceIndex( record.getMateReferenceIndex() );
-                otherRec.setMateAlignmentStart( record.getMateAlignmentStart() );
+            otherRec.setReadName(record.getReadName());
+            otherRec.setReadBases(record.getReadBases());
+            otherRec.setBaseQualities(record.getBaseQualities());
+            if (record.getReadPairedFlag() && !record.getMateUnmappedFlag()) {
+                otherRec.setMateReferenceIndex(record.getMateReferenceIndex());
+                otherRec.setMateAlignmentStart(record.getMateAlignmentStart());
             }
 
 
             /* get reference sequence */
-            final int tid = record.getHeader().getSequenceIndex( commaStrs[0] );
-            if( tid == -1 ) throw new SAMException("Unknown contig in " + semiColonStr);
-            otherRec.setReferenceIndex( tid );
+            final int tid = record.getHeader().getSequenceIndex(commaStrs[0]);
+            if (tid == -1)
+                throw new SAMException("Unknown contig in " + semiColonStr + ". Record: " + record);
+            otherRec.setReferenceIndex(tid);
 
             /* fill POS */
             final int alignStart;
             try {
                 alignStart = Integer.parseInt(commaStrs[1]);
-            } catch( final NumberFormatException err ) {
-                throw new SAMException("bad POS in "+semiColonStr, err);
+            } catch (final NumberFormatException err) {
+                throw new SAMException("bad POS in " + semiColonStr + ". Record: " + record, err);
             }
 
-            otherRec.setAlignmentStart( alignStart );
+            otherRec.setAlignmentStart(alignStart);
 
             /* set TLEN */
-            if( record.getReadPairedFlag() &&
-                !record.getMateUnmappedFlag() &&
-                record.getMateReferenceIndex() == tid ) {
-                otherRec.setInferredInsertSize( record.getMateAlignmentStart() - alignStart );
+            if (record.getReadPairedFlag() &&
+                    !record.getMateUnmappedFlag() &&
+                    record.getMateReferenceIndex() == tid) {
+                otherRec.setInferredInsertSize(record.getMateAlignmentStart() - alignStart);
             }
 
             /* set FLAG */
-           int other_flag = record_flag;
-           other_flag |= (commaStrs[2].equals("+") ? 0 : SAMFlag.READ_REVERSE_STRAND.flag) ;
+            int other_flag = record_flag;
+            other_flag |= (commaStrs[2].equals("+") ? 0 : SAMFlag.READ_REVERSE_STRAND.flag);
            /* spec: Conventionally, at a supplementary line, the  1st element points to the primary line */
-           if( !( record.getSupplementaryAlignmentFlag() && i==0 ) ) {
-               other_flag |= SAMFlag.SUPPLEMENTARY_ALIGNMENT.flag;
-           }
-           otherRec.setFlags(other_flag);
+            if (!(record.getSupplementaryAlignmentFlag() && i == 0)) {
+                other_flag |= SAMFlag.SUPPLEMENTARY_ALIGNMENT.flag;
+            }
+            otherRec.setFlags(other_flag);
 
            /* set CIGAR */
-           otherRec.setCigar( TextCigarCodec.decode( commaStrs[3] ) );
+            otherRec.setCigar(TextCigarCodec.decode(commaStrs[3]));
 
             /* set MAPQ */
             try {
-                otherRec.setMappingQuality( Integer.parseInt(commaStrs[4]) );
+                otherRec.setMappingQuality(Integer.parseInt(commaStrs[4]));
             } catch (final NumberFormatException err) {
-                throw new SAMException("bad MAPQ in "+semiColonStr, err);
+                throw new SAMException("bad MAPQ in " + semiColonStr + ". Record: " + record, err);
             }
 
             /* fill NM */
@@ -1210,16 +1223,16 @@ public final class SAMUtils {
                     otherRec.setAttribute(SAMTagUtil.getSingleton().NM, Integer.parseInt(commaStrs[5]));
                 }
             } catch (final NumberFormatException err) {
-                throw new SAMException("bad NM in "+semiColonStr, err);
+                throw new SAMException("bad NM in " + semiColonStr + ". Record: " + record, err);
             }
 
             /* if strand is not the same: reverse-complement */
-            if( otherRec.getReadNegativeStrandFlag() != record.getReadNegativeStrandFlag() ) {
-                SAMRecordUtil.reverseComplement(otherRec);
+            if (otherRec.getReadNegativeStrandFlag() != record.getReadNegativeStrandFlag()) {
+                otherRec.reverseComplement(true);
             }
 
             /* add the alignment */
-            alignments.add( otherRec );
+            alignments.add(otherRec);
         }
         return alignments;
     }
