@@ -164,6 +164,7 @@ public interface SamReader extends Iterable<SAMRecord>, Closeable {
      * Only a single open iterator on a SAM or BAM file may be extant at any one time.  If you want to start
      * a second iteration, the first one must be closed first.
      */
+    @Override
     public SAMRecordIterator iterator();
 
     /**
@@ -381,7 +382,11 @@ public interface SamReader extends Iterable<SAMRecord>, Closeable {
             this.resource = resource;
         }
 
-        PrimitiveSamReader underlyingReader() {
+        /**
+         * Access the underlying {@link PrimitiveSamReader} used by this adapter.
+         * @return the {@link PrimitiveSamReader} used by this adapter.
+         */
+        public PrimitiveSamReader underlyingReader() {
             return p;
         }
 
@@ -554,6 +559,7 @@ public interface SamReader extends Iterable<SAMRecord>, Closeable {
             wrappedIterator = iterator;
         }
 
+        @Override
         public SAMRecordIterator assertSorted(final SAMFileHeader.SortOrder sortOrder) {
 
             if (sortOrder == null || sortOrder == SAMFileHeader.SortOrder.unsorted) {
@@ -565,6 +571,7 @@ public interface SamReader extends Iterable<SAMRecord>, Closeable {
             return this;
         }
 
+        @Override
         public SAMRecord next() {
             final SAMRecord result = wrappedIterator.next();
             if (comparator != null) {
@@ -587,10 +594,13 @@ public interface SamReader extends Iterable<SAMRecord>, Closeable {
             return result;
         }
 
+        @Override
         public void close() { wrappedIterator.close(); }
 
+        @Override
         public boolean hasNext() { return wrappedIterator.hasNext(); }
 
+        @Override
         public void remove() { wrappedIterator.remove(); }
     }
 
