@@ -182,6 +182,7 @@ public abstract class AbstractLocusIterator<T extends AbstractRecordAndOffset, K
      * @return iterator over all/all covered locus position in reference according to <code>emitUncoveredLoci</code>
      * value.
      */
+    @Override
     public Iterator<K> iterator() {
         if (samIterator != null) {
             throw new IllegalStateException("Cannot call iterator() more than once on " + this.getClass().getSimpleName());
@@ -202,6 +203,7 @@ public abstract class AbstractLocusIterator<T extends AbstractRecordAndOffset, K
     /**
      * Closes inner <code>SamIterator</>.
      */
+    @Override
     public void close() {
         this.samIterator.close();
     }
@@ -216,6 +218,7 @@ public abstract class AbstractLocusIterator<T extends AbstractRecordAndOffset, K
      * 2) there are AbstractLocusInfo<T>s in some stage of accumulation
      * 3) there are loci in the target mask that have yet to be accumulated (even if there are no reads covering them)
      */
+    @Override
     public boolean hasNext() {
         if (this.samIterator == null) {
             iterator();
@@ -253,6 +256,7 @@ public abstract class AbstractLocusIterator<T extends AbstractRecordAndOffset, K
      *
      * @return information about next locus position in reference sequence
      */
+    @Override
     public K next() {
         // if we don't have any completed entries to return, try and make some!
         while (complete.isEmpty() && samHasMore()) {
@@ -434,10 +438,8 @@ public abstract class AbstractLocusIterator<T extends AbstractRecordAndOffset, K
     private void populateCompleteQueue(final Locus stopBeforeLocus) {
         // Because of gapped alignments, it is possible to create LocusInfo's with no reads associated with them.
         // Skip over these if not including indels
-        while (!accumulator.isEmpty() && accumulator.get(0).isEmpty() &&
-                locusComparator.compare(accumulator.get(0), stopBeforeLocus) < 0) {
-            accumulator.remove(0);
-        }
+        removeSkippedRegion(stopBeforeLocus);
+
         if (accumulator.isEmpty()) {
             return;
         }
@@ -471,10 +473,22 @@ public abstract class AbstractLocusIterator<T extends AbstractRecordAndOffset, K
         lastPosition = locusInfo.getPosition();
     }
 
+    private void removeSkippedRegion(Locus stopBeforeLocus) {
+        int i = 0;
+        while (i < accumulator.size() && accumulator.get(i).isEmpty() &&
+                locusComparator.compare(accumulator.get(i), stopBeforeLocus) < 0) {
+            i++;
+        }
+        if (i > 0){
+            accumulator.subList(0, i).clear();
+        }
+    }
+
     protected SAMSequenceRecord getReferenceSequence(final int referenceSequenceIndex) {
         return samReader.getFileHeader().getSequence(referenceSequenceIndex);
     }
 
+    @Override
     public void remove() {
         throw new UnsupportedOperationException("Can not remove records from a SAM file via an iterator!");
     }
