@@ -30,22 +30,25 @@ import htsjdk.samtools.seekablestream.SeekableBufferedStream;
 import htsjdk.samtools.seekablestream.SeekableFileStream;
 import htsjdk.samtools.seekablestream.SeekableHTTPStream;
 import htsjdk.samtools.seekablestream.SeekableStream;
+import htsjdk.samtools.util.zip.InflaterFactory;
 
-import java.io.ByteArrayOutputStream;
-import java.io.File;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.RandomAccessFile;
+import java.io.*;
 import java.net.URL;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
+import java.nio.channels.SeekableByteChannel;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.StandardOpenOption;
 import java.util.Arrays;
 
-/*
+/**
  * Utility class for reading BGZF block compressed files.  The caller can treat this file like any other InputStream.
  * It probably is not necessary to wrap this stream in a buffering stream, because there is internal buffering.
  * The advantage of BGZF over conventional GZip format is that BGZF allows for seeking without having to read the
- * entire file up to the location being sought.  Note that seeking is only possible if the ctor(File) is used.
+ * entire file up to the location being sought.  Note that seeking is only possible if the input stream is seekable.
+ * 
+ * Note that this implementation is not synchronized. If multiple threads access an instance concurrently, it must be synchronized externally. 
  *
  * c.f. http://samtools.sourceforge.net/SAM1.pdf for details of BGZF format
  */
@@ -54,30 +57,52 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
     public final static String INCORRECT_HEADER_SIZE_MSG = "Incorrect header size for file: ";
     public final static String UNEXPECTED_BLOCK_LENGTH_MSG = "Unexpected compressed block length: ";
     public final static String PREMATURE_END_MSG = "Premature end of file: ";
-    public final static String CANNOT_SEEK_STREAM_MSG = "Cannot seek on stream based file ";
+    public final static String CANNOT_SEEK_STREAM_MSG = "Cannot seek a position for a non-file stream";
+    public final static String CANNOT_SEEK_CLOSED_STREAM_MSG = "Cannot seek a position for a closed stream";
     public final static String INVALID_FILE_PTR_MSG = "Invalid file pointer: ";
 
     private InputStream mStream = null;
+    private boolean mIsClosed = false;
     private SeekableStream mFile = null;
     private byte[] mFileBuffer = null;
-    private byte[] mCurrentBlock = null;
+    private DecompressedBlock mCurrentBlock = null;
     private int mCurrentOffset = 0;
-    private long mBlockAddress = 0;
-    private int mLastBlockLength = 0;
-    private final BlockGunzipper blockGunzipper = new BlockGunzipper();
-
+    private long mStreamOffset = 0;
+    private final BlockGunzipper blockGunzipper;
 
     /**
      * Note that seek() is not supported if this ctor is used.
+     * @param stream source of bytes
      */
     public BlockCompressedInputStream(final InputStream stream) {
-        this(stream, true);
+        this(stream, true, BlockGunzipper.getDefaultInflaterFactory());
     }
 
     /**
      * Note that seek() is not supported if this ctor is used.
+     * @param stream source of bytes
+     * @param inflaterFactory {@link InflaterFactory} used by {@link BlockGunzipper}
+     */
+    public BlockCompressedInputStream(final InputStream stream, final InflaterFactory inflaterFactory) {
+        this(stream, true, inflaterFactory);
+    }
+
+    /**
+     * Note that seek() is not supported if this ctor is used.
+     * @param stream source of bytes
+     * @param allowBuffering if true, allow buffering
      */
     public BlockCompressedInputStream(final InputStream stream, final boolean allowBuffering) {
+        this(stream, allowBuffering, BlockGunzipper.getDefaultInflaterFactory());
+    }
+
+    /**
+     * Note that seek() is not supported if this ctor is used.
+     * @param stream source of bytes
+     * @param allowBuffering if true, allow buffering
+     * @param inflaterFactory {@link InflaterFactory} used by {@link BlockGunzipper}
+     */
+    public BlockCompressedInputStream(final InputStream stream, final boolean allowBuffering, final InflaterFactory inflaterFactory) {
         if (allowBuffering) {
             mStream = IOUtil.toBufferedStream(stream);
         }
@@ -86,30 +111,68 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
         }
 
         mFile = null;
+        blockGunzipper = new BlockGunzipper(inflaterFactory);
     }
 
     /**
      * Use this ctor if you wish to call seek()
+     * @param file source of bytes
+     * @throws IOException
      */
     public BlockCompressedInputStream(final File file) throws IOException {
-        mFile = new SeekableFileStream(file);
-        mStream = null;
-
+        this(file, BlockGunzipper.getDefaultInflaterFactory());
     }
 
+    /**
+     * Use this ctor if you wish to call seek()
+     * @param file source of bytes
+     * @param inflaterFactory {@link InflaterFactory} used by {@link BlockGunzipper}
+     * @throws IOException
+     */
+    public BlockCompressedInputStream(final File file, final InflaterFactory inflaterFactory) throws IOException {
+        mFile = new SeekableFileStream(file);
+        mStream = null;
+        blockGunzipper = new BlockGunzipper(inflaterFactory);
+    }
+
+    /**
+     * @param url source of bytes
+     */
     public BlockCompressedInputStream(final URL url) {
+        this(url, BlockGunzipper.getDefaultInflaterFactory());
+    }
+
+    /**
+     * @param url source of bytes
+     * @param inflaterFactory {@link InflaterFactory} used by {@link BlockGunzipper}
+     */
+    public BlockCompressedInputStream(final URL url, final InflaterFactory inflaterFactory) {
         mFile = new SeekableBufferedStream(new SeekableHTTPStream(url));
         mStream = null;
+        blockGunzipper = new BlockGunzipper(inflaterFactory);
     }
 
     /**
      * For providing some arbitrary data source.  No additional buffering is
      * provided, so if the underlying source is not buffered, wrap it in a
      * SeekableBufferedStream before passing to this ctor.
+     * @param strm source of bytes
      */
     public BlockCompressedInputStream(final SeekableStream strm) {
+        this(strm, BlockGunzipper.getDefaultInflaterFactory());
+    }
+
+    /**
+     * For providing some arbitrary data source.  No additional buffering is
+     * provided, so if the underlying source is not buffered, wrap it in a
+     * SeekableBufferedStream before passing to this ctor.
+     * @param strm source of bytes
+     * @param inflaterFactory {@link InflaterFactory} used by {@link BlockGunzipper}
+     */
+    public BlockCompressedInputStream(final SeekableStream strm, final InflaterFactory inflaterFactory) {
         mFile = strm;
         mStream = null;
+        blockGunzipper = new BlockGunzipper(inflaterFactory);
     }
 
     /**
@@ -127,14 +190,15 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      * Note that although the next caller can read this many bytes without blocking, the available() method call itself
      * may block in order to fill an internal buffer if it has been exhausted.
      */
+    @Override
     public int available() throws IOException {
-        if (mCurrentBlock == null || mCurrentOffset == mCurrentBlock.length) {
+        if (mCurrentBlock == null || mCurrentOffset == mCurrentBlock.mBlock.length) {
             readBlock();
         }
         if (mCurrentBlock == null) {
             return 0;
         }
-        return mCurrentBlock.length - mCurrentOffset;
+        return mCurrentBlock.mBlock.length - mCurrentOffset;
     }
 
     /**
@@ -142,12 +206,13 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      * <code>false</code> otherwise.
      */
     public boolean endOfBlock() {
-        return (mCurrentBlock != null && mCurrentOffset == mCurrentBlock.length);
+        return (mCurrentBlock != null && mCurrentOffset == mCurrentBlock.mBlock.length);
     }
 
     /**
      * Closes the underlying InputStream or RandomAccessFile
      */
+    @Override
     public void close() throws IOException {
         if (mFile != null) {
             mFile.close();
@@ -159,6 +224,9 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
         // Encourage garbage collection
         mFileBuffer = null;
         mCurrentBlock = null;
+
+        // Mark as closed
+        mIsClosed = true;
     }
 
     /**
@@ -168,8 +236,9 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
 
      * @return the next byte of data, or -1 if the end of the stream is reached.
      */
+    @Override
     public int read() throws IOException {
-        return (available() > 0) ? (mCurrentBlock[mCurrentOffset++] & 0xFF) : -1;
+        return (available() > 0) ? (mCurrentBlock.mBlock[mCurrentOffset++] & 0xFF) : -1;
     }
 
     /**
@@ -183,6 +252,7 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      * @return the total number of bytes read into the buffer, or -1 is there is no more data because the end of
      * the stream has been reached.
      */
+    @Override
     public int read(final byte[] buffer) throws IOException {
         return read(buffer, 0, buffer.length);
     }
@@ -199,48 +269,47 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      *          character, or null if the end of the stream has been reached
      *
      * @exception  IOException  If an I/O error occurs
-     * @
      */
     public String readLine() throws IOException {
-    	int available = available();
+        int available = available();
         if (available == 0) {
             return null;
         }
         if(null == buf){ // lazy initialisation 
-        	buf = new ByteArrayOutputStream(8192);
+            buf = new ByteArrayOutputStream(8192);
         }
         buf.reset();
-    	boolean done = false;
-    	boolean foundCr = false; // \r found flag
+        boolean done = false;
+        boolean foundCr = false; // \r found flag
         while (!done) {
-        	int linetmpPos = mCurrentOffset;
-        	int bCnt = 0;
-        	while((available-- > 0)){
-        		final byte c = mCurrentBlock[linetmpPos++];
-        		if(c == eol){ // found \n
-        			done = true;
-        			break;
-        		} else if(foundCr){  // previous char was \r
-        			--linetmpPos; // current char is not \n so put it back
-        			done = true;
-        			break;
-        		} else if(c == eolCr){ // found \r
-					foundCr = true;
-        			continue; // no ++bCnt
-        		}
-				++bCnt;
-        	}
-        	if(mCurrentOffset < linetmpPos){
-				buf.write(mCurrentBlock, mCurrentOffset, bCnt);
-	        	mCurrentOffset = linetmpPos;
-        	}
-        	available = available();    
-        	if(available == 0){
-        		// EOF
-        		done = true;
-        	}
+            int linetmpPos = mCurrentOffset;
+            int bCnt = 0;
+            while((available-- > 0)){
+                final byte c = mCurrentBlock.mBlock[linetmpPos++];
+                if(c == eol){ // found \n
+                    done = true;
+                    break;
+                } else if(foundCr){  // previous char was \r
+                    --linetmpPos; // current char is not \n so put it back
+                    done = true;
+                    break;
+                } else if(c == eolCr){ // found \r
+                    foundCr = true;
+                    continue; // no ++bCnt
+                }
+                ++bCnt;
+            }
+            if(mCurrentOffset < linetmpPos) {
+                buf.write(mCurrentBlock.mBlock, mCurrentOffset, bCnt);
+                mCurrentOffset = linetmpPos;
+            }
+            available = available();
+            if(available == 0) {
+                // EOF
+                done = true;
+            }
         }
-    	return buf.toString();
+        return buf.toString();
     }
 
     /**
@@ -255,6 +324,7 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      * @return the total number of bytes read into the buffer, or -1 if there is no more data because the end of
      * the stream has been reached.
      */
+    @Override
     public int read(final byte[] buffer, int offset, int length) throws IOException {
         final int originalLength = length;
         while (length > 0) {
@@ -267,7 +337,7 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
                 break;
             }
             final int copyLength = Math.min(length, available);
-            System.arraycopy(mCurrentBlock, mCurrentOffset, buffer, offset, copyLength);
+            System.arraycopy(mCurrentBlock.mBlock, mCurrentOffset, buffer, offset, copyLength);
             mCurrentOffset += copyLength;
             offset += copyLength;
             length -= copyLength;
@@ -279,40 +349,57 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      * Seek to the given position in the file.  Note that pos is a special virtual file pointer,
      * not an actual byte offset.
      *
-     * @param pos virtual file pointer
+     * @param pos virtual file pointer position
+     * @throws IOException if stream is closed or not a file based stream
      */
     public void seek(final long pos) throws IOException {
+        // Must be before the mFile == null check because mFile == null for closed files and streams
+        if (mIsClosed) {
+            throw new IOException(CANNOT_SEEK_CLOSED_STREAM_MSG);
+        }
+
+        // Cannot seek on streams that are not file based
         if (mFile == null) {
             throw new IOException(CANNOT_SEEK_STREAM_MSG);
         }
+
         // Decode virtual file pointer
-        // Upper 48 bits is the byte offset into the compressed stream of a block.
-        // Lower 16 bits is the byte offset into the uncompressed stream inside the block.
+        // Upper 48 bits is the byte offset into the compressed stream of a
+        // block.
+        // Lower 16 bits is the byte offset into the uncompressed stream inside
+        // the block.
         final long compressedOffset = BlockCompressedFilePointerUtil.getBlockAddress(pos);
         final int uncompressedOffset = BlockCompressedFilePointerUtil.getBlockOffset(pos);
         final int available;
-        if (mBlockAddress == compressedOffset && mCurrentBlock != null) {
-            available = mCurrentBlock.length;
+        if (mCurrentBlock != null && mCurrentBlock.mBlockAddress == compressedOffset) {
+            available = mCurrentBlock.mBlock.length;
         } else {
+            prepareForSeek();
             mFile.seek(compressedOffset);
-            mBlockAddress = compressedOffset;
-            mLastBlockLength = 0;
-            readBlock();
+            mStreamOffset = compressedOffset;
+            mCurrentBlock = nextBlock(getBufferForReuse(mCurrentBlock));
+            mCurrentOffset = 0;
             available = available();
         }
-        if (uncompressedOffset > available ||
-                (uncompressedOffset == available && !eof())) {
-            throw new IOException(INVALID_FILE_PTR_MSG + pos + " for " + mFile.getSource());
+        if (uncompressedOffset > available || (uncompressedOffset == available && !eof())) {
+            throw new IOException(INVALID_FILE_PTR_MSG + pos + " for " + getSource());
         }
         mCurrentOffset = uncompressedOffset;
     }
+    
+    /**
+     * Performs cleanup required before seek is called on the underlying stream 
+     */
+    protected void prepareForSeek() {
+    }
 
     private boolean eof() throws IOException {
         if (mFile.eof()) {
             return true;
         }
         // If the last remaining block is the size of the EMPTY_GZIP_BLOCK, this is the same as being at EOF.
-        return (mFile.length() - (mBlockAddress + mLastBlockLength) == BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length);
+        return (mFile.length() - (mCurrentBlock.mBlockAddress
+                + mCurrentBlock.mBlockCompressedSize) == BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length);
     }
 
     /**
@@ -321,12 +408,17 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
      * the two.
      */
     public long getFilePointer() {
-        if (mCurrentOffset == mCurrentBlock.length) {
-            // If current offset is at the end of the current block, file pointer should point
-            // to the beginning of the next block.
-            return BlockCompressedFilePointerUtil.makeFilePointer(mBlockAddress + mLastBlockLength, 0);
+        if (mCurrentBlock == null) {
+            // Haven't read anything yet = at start of stream
+            return BlockCompressedFilePointerUtil.makeFilePointer(0, 0);
         }
-        return BlockCompressedFilePointerUtil.makeFilePointer(mBlockAddress, mCurrentOffset);
+        if (mCurrentOffset > 0 && mCurrentOffset == mCurrentBlock.mBlock.length) {
+            // If current offset is at the end of the current block, file
+            // pointer should point
+            // to the beginning of the next block.
+            return BlockCompressedFilePointerUtil.makeFilePointer(mCurrentBlock.mBlockAddress + mCurrentBlock.mBlockCompressedSize, 0);
+        }
+        return BlockCompressedFilePointerUtil.makeFilePointer(mCurrentBlock.mBlockAddress, mCurrentOffset);
     }
 
     @Override
@@ -363,49 +455,104 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
     }
 
     private void readBlock() throws IOException {
-
+        mCurrentBlock = nextBlock(getBufferForReuse(mCurrentBlock));
+        mCurrentOffset = 0;
+        checkAndRethrowDecompressionException();
+    }
+    /**
+     * Reads and decompresses the next block
+     * @param bufferAvailableForReuse decompression buffer available for reuse
+     * @return next block in the decompressed stream
+     */
+    protected DecompressedBlock nextBlock(byte[] bufferAvailableForReuse) {
+        return processNextBlock(bufferAvailableForReuse);
+    }
+    /**
+     * Rethrows an exception encountered during decompression
+     * @throws IOException
+     */
+    private void checkAndRethrowDecompressionException() throws IOException {
+        if (mCurrentBlock.mException != null) {
+            if (mCurrentBlock.mException instanceof IOException) {
+                throw (IOException) mCurrentBlock.mException;
+            } else if (mCurrentBlock.mException instanceof RuntimeException) {
+                throw (RuntimeException) mCurrentBlock.mException;
+            } else {
+                throw new RuntimeException(mCurrentBlock.mException);
+            }
+        }
+    }
+    
+    /**
+     * Attempt to reuse the buffer of the given block 
+     * @param block owning block
+     * @return null decompressing buffer to reuse, null if no buffer is available
+     */
+    private byte[] getBufferForReuse(DecompressedBlock block) {
+        if (block == null) return null;
+        return block.mBlock;
+    }
+    
+    /**
+     * Decompress the next block from the input stream. When using asynchronous 
+     * IO, this will be called by the background thread.
+     * @param bufferAvailableForReuse buffer in which to place decompressed block. A null or
+     *  incorrectly sized buffer will result in the buffer being ignored and
+     *  a new buffer allocated for decompression.
+     * @return next block in input stream
+     */
+    protected DecompressedBlock processNextBlock(byte[] bufferAvailableForReuse) {
         if (mFileBuffer == null) {
             mFileBuffer = new byte[BlockCompressedStreamConstants.MAX_COMPRESSED_BLOCK_SIZE];
         }
-        int count = readBytes(mFileBuffer, 0, BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH);
-        if (count == 0) {
-            // Handle case where there is no empty gzip block at end.
-            mCurrentOffset = 0;
-            mBlockAddress += mLastBlockLength;
-            mCurrentBlock = new byte[0];
-            return;
+        long blockAddress = mStreamOffset;
+        try {
+            final int headerByteCount = readBytes(mFileBuffer, 0, BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH);
+            mStreamOffset += headerByteCount;
+            if (headerByteCount == 0) {
+                // Handle case where there is no empty gzip block at end.
+                return new DecompressedBlock(blockAddress, new byte[0], 0);
+            }
+            if (headerByteCount != BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH) {
+                return new DecompressedBlock(blockAddress, headerByteCount, new IOException(INCORRECT_HEADER_SIZE_MSG + getSource()));
+            }
+            final int blockLength = unpackInt16(mFileBuffer, BlockCompressedStreamConstants.BLOCK_LENGTH_OFFSET) + 1;
+            if (blockLength < BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH || blockLength > mFileBuffer.length) {
+                return new DecompressedBlock(blockAddress, blockLength,
+                        new IOException(UNEXPECTED_BLOCK_LENGTH_MSG + blockLength + " for " + getSource()));
+            }
+            final int remaining = blockLength - BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH;
+            final int dataByteCount = readBytes(mFileBuffer, BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH,
+                    remaining);
+            mStreamOffset += dataByteCount;
+            if (dataByteCount != remaining) {
+                return new DecompressedBlock(blockAddress, blockLength,
+                        new FileTruncatedException(PREMATURE_END_MSG + getSource()));
+            }
+            final byte[] decompressed = inflateBlock(mFileBuffer, blockLength, bufferAvailableForReuse);
+            return new DecompressedBlock(blockAddress, decompressed, blockLength);
+        } catch (IOException e) {
+            return new DecompressedBlock(blockAddress, 0, e);
         }
-        if (count != BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH) {
-            throw new IOException(INCORRECT_HEADER_SIZE_MSG + mFile.getSource());
-        }
-        final int blockLength = unpackInt16(mFileBuffer, BlockCompressedStreamConstants.BLOCK_LENGTH_OFFSET) + 1;
-        if (blockLength < BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH || blockLength > mFileBuffer.length) {
-            throw new IOException(UNEXPECTED_BLOCK_LENGTH_MSG + blockLength + " for " + mFile.getSource());
-        }
-        final int remaining = blockLength - BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH;
-        count = readBytes(mFileBuffer, BlockCompressedStreamConstants.BLOCK_HEADER_LENGTH, remaining);
-        if (count != remaining) {
-            throw new FileTruncatedException(PREMATURE_END_MSG + mFile.getSource());
-        }
-        inflateBlock(mFileBuffer, blockLength);
-        mCurrentOffset = 0;
-        mBlockAddress += mLastBlockLength;
-        mLastBlockLength = blockLength;
     }
 
-    private void inflateBlock(final byte[] compressedBlock, final int compressedLength) throws IOException {
-        final int uncompressedLength = unpackInt32(compressedBlock, compressedLength-4);
-        byte[] buffer = mCurrentBlock;
-        mCurrentBlock = null;
-        if (buffer == null || buffer.length != uncompressedLength) {
-            try {
-                buffer = new byte[uncompressedLength];
-            } catch (final NegativeArraySizeException e) {
-                throw new RuntimeIOException(mFile.getSource() + " has invalid uncompressedLength: " + uncompressedLength, e);
-            }
+    private byte[] inflateBlock(final byte[] compressedBlock, final int compressedLength,
+            final byte[] bufferAvailableForReuse) throws IOException {
+        final int uncompressedLength = unpackInt32(compressedBlock, compressedLength - 4);
+        if (uncompressedLength < 0) {
+            throw new RuntimeIOException(getSource() + " has invalid uncompressedLength: " + uncompressedLength);
+        }
+        byte[] buffer = bufferAvailableForReuse;
+        if (buffer == null || uncompressedLength != buffer.length) {
+        	// can't reuse the buffer since the size is incorrect
+            buffer = new byte[uncompressedLength];
         }
         blockGunzipper.unzipBlock(buffer, compressedBlock, compressedLength);
-        mCurrentBlock = buffer;
+        return buffer;
+    }
+
+    private String getSource() {
+        return mFile == null ? "data stream" : mFile.getSource();
     }
 
     private int readBytes(final byte[] buffer, final int offset, final int length) throws IOException {
@@ -456,41 +603,98 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
 
     public enum FileTermination {HAS_TERMINATOR_BLOCK, HAS_HEALTHY_LAST_BLOCK, DEFECTIVE}
 
+    /**
+     *
+     * @param file the file to check
+     * @return status of the last compressed block
+     * @throws IOException
+     */
     public static FileTermination checkTermination(final File file) throws IOException {
-        final long fileSize = file.length();
+        return checkTermination(file == null ? null : file.toPath());
+    }
+
+    /**
+     *
+     * @param path to the file to check
+     * @return status of the last compressed block
+     * @throws IOException
+     */
+    public static FileTermination checkTermination(final Path path) throws IOException {
+        try( final SeekableByteChannel channel = Files.newByteChannel(path, StandardOpenOption.READ) ){
+            return checkTermination(channel);
+        }
+    }
+
+    /**
+     * check the status of the final bzgipped block for the given bgzipped resource
+     *
+     * @param channel an open channel to read from,
+     * the channel will remain open and the initial position will be restored when the operation completes
+     * this makes no guarantee about the state of the channel if an exception is thrown during reading
+     *
+     * @return the status of the last compressed black
+     * @throws IOException
+     */
+    public static FileTermination checkTermination(SeekableByteChannel channel) throws IOException {
+        final long fileSize = channel.size();
         if (fileSize < BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length) {
             return FileTermination.DEFECTIVE;
         }
-        final RandomAccessFile raFile = new RandomAccessFile(file, "r");
+        final long initialPosition = channel.position();
+        boolean exceptionThrown = false;
         try {
-            raFile.seek(fileSize - BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length);
-            byte[] buf = new byte[BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length];
-            raFile.readFully(buf);
-            if (Arrays.equals(buf, BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK)) {
+            channel.position(fileSize - BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length);
+
+            //Check if the end of the file is an empty gzip block which is used as the terminator for a bgzipped file
+            final ByteBuffer lastBlockBuffer = ByteBuffer.allocate(BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length);
+            readFully(channel, lastBlockBuffer);
+            if (Arrays.equals(lastBlockBuffer.array(), BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK)) {
                 return FileTermination.HAS_TERMINATOR_BLOCK;
             }
-            final int bufsize = (int)Math.min(fileSize, BlockCompressedStreamConstants.MAX_COMPRESSED_BLOCK_SIZE);
-            buf = new byte[bufsize];
-            raFile.seek(fileSize - bufsize);
-            raFile.read(buf);
-            for (int i = buf.length - BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length;
-                    i >= 0; --i) {
+
+            //if the last block isn't an empty gzip block, check to see if it is a healthy compressed block or if it's corrupted
+            final int bufsize = (int) Math.min(fileSize, BlockCompressedStreamConstants.MAX_COMPRESSED_BLOCK_SIZE);
+            final byte[] bufferArray = new byte[bufsize];
+            channel.position(fileSize - bufsize);
+            readFully(channel, ByteBuffer.wrap(bufferArray));
+            for (int i = bufferArray.length - BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK.length;
+                 i >= 0; --i) {
                 if (!preambleEqual(BlockCompressedStreamConstants.GZIP_BLOCK_PREAMBLE,
-                        buf, i, BlockCompressedStreamConstants.GZIP_BLOCK_PREAMBLE.length)) {
+                                   bufferArray, i, BlockCompressedStreamConstants.GZIP_BLOCK_PREAMBLE.length)) {
                     continue;
                 }
-                final ByteBuffer byteBuffer = ByteBuffer.wrap(buf, i + BlockCompressedStreamConstants.GZIP_BLOCK_PREAMBLE.length, 4);
+                final ByteBuffer byteBuffer = ByteBuffer.wrap(bufferArray,
+                                                              i + BlockCompressedStreamConstants.GZIP_BLOCK_PREAMBLE.length,
+                                                              4);
                 byteBuffer.order(ByteOrder.LITTLE_ENDIAN);
-                final int totalBlockSizeMinusOne =  byteBuffer.getShort() & 0xFFFF;
-                if (buf.length - i == totalBlockSizeMinusOne + 1) {
+                final int totalBlockSizeMinusOne = byteBuffer.getShort() & 0xFFFF;
+                if (bufferArray.length - i == totalBlockSizeMinusOne + 1) {
                     return FileTermination.HAS_HEALTHY_LAST_BLOCK;
                 } else {
                     return FileTermination.DEFECTIVE;
                 }
             }
             return FileTermination.DEFECTIVE;
+        } catch (final Throwable e) {
+            exceptionThrown = true;
+            throw e;
         } finally {
-            raFile.close();
+            //if an exception was thrown we don't want to reset the position because that would be likely to throw again
+            //and suppress the initial exception
+            if(!exceptionThrown) {
+                channel.position(initialPosition);
+            }
+        }
+    }
+
+    /**
+     * read as many bytes as dst's capacity into dst or throw if that's not possible
+     * @throws EOFException if channel has fewer bytes available than dst's capacity
+     */
+    static void readFully(SeekableByteChannel channel, ByteBuffer dst) throws IOException {
+        final int bytesRead = channel.read(dst);
+        if (bytesRead < dst.capacity()){
+            throw new EOFException();
         }
     }
 
@@ -508,6 +712,38 @@ public class BlockCompressedInputStream extends InputStream implements LocationA
         }
         return true;
     }
+
+    protected static class DecompressedBlock {
+        /**
+         * Decompressed block
+         */
+        private final byte[] mBlock;
+        /**
+         * Compressed size of block (the uncompressed size can be found using
+         * mBlock.length)
+         */
+        private final int mBlockCompressedSize;
+        /**
+         * Stream offset of start of block
+         */
+        private final long mBlockAddress;
+        /**
+         * Exception thrown (if any) when attempting to decompress block
+         */
+        private final Exception mException;
+
+        public DecompressedBlock(long blockAddress, byte[] block, int compressedSize) {
+            mBlock = block;
+            mBlockAddress = blockAddress;
+            mBlockCompressedSize = compressedSize;
+            mException = null;
+        }
+
+        public DecompressedBlock(long blockAddress, int compressedSize, Exception exception) {
+            mBlock = new byte[0];
+            mBlockAddress = blockAddress;
+            mBlockCompressedSize = compressedSize;
+            mException = exception;
+        }
+    }
 }
-
-
