@@ -28,6 +28,7 @@ import htsjdk.samtools.util.zip.DeflaterFactory;
 import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
+import java.nio.file.Files;
 import java.util.zip.CRC32;
 import java.util.zip.Deflater;
 
@@ -282,7 +283,7 @@ public class BlockCompressedOutputStream
         codec.writeBytes(BlockCompressedStreamConstants.EMPTY_GZIP_BLOCK);
         codec.close();
         // Can't re-open something that is not a regular file, e.g. a named pipe or an output stream
-        if (this.file == null || !this.file.isFile()) return;
+        if (this.file == null || !this.file.isFile() || !Files.isRegularFile(this.file.toPath())) return;
         if (BlockCompressedInputStream.checkTermination(this.file) !=
                 BlockCompressedInputStream.FileTermination.HAS_TERMINATOR_BLOCK) {
             throw new IOException("Terminator block not found after closing BGZF file " + this.file);
@@ -296,6 +297,7 @@ public class BlockCompressedOutputStream
      * @param bite
      * @throws IOException
      */
+    @Override
     public void write(final int bite) throws IOException {
         singleByteArray[0] = (byte)bite;
         write(singleByteArray);
