@@ -1,7 +1,7 @@
 /*
  * The MIT License
  *
- * Copyright (c) 2009 The Broad Institute
+ * Copyright (c) 2018 The Broad Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,14 +26,15 @@ package htsjdk.samtools.util;
 import htsjdk.samtools.Defaults;
 
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.Serializable;
 import java.lang.reflect.Array;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -96,7 +97,7 @@ public class SortingCollection<T> implements Iterable<T> {
     }
 
     /** Directories where files of sorted records go. */
-    private final File[] tmpDirs;
+    private final Path[] tmpDirs;
 
     /** The minimum amount of space free on a temp filesystem to write a file there. */
     private final long TMP_SPACE_FREE = IOUtil.FIVE_GBS;
@@ -124,7 +125,7 @@ public class SortingCollection<T> implements Iterable<T> {
     /**
      * List of files in tmpDir containing sorted records
      */
-    private final List<File> files = new ArrayList<File>();
+    private final List<Path> files = new ArrayList<>();
 
     private boolean destructiveIteration = true;
 
@@ -139,7 +140,7 @@ public class SortingCollection<T> implements Iterable<T> {
      * @param tmpDir Where to write files of records that will not fit in RAM
      */
     private SortingCollection(final Class<T> componentType, final SortingCollection.Codec<T> codec,
-                             final Comparator<T> comparator, final int maxRecordsInRam, final File... tmpDir) {
+                             final Comparator<T> comparator, final int maxRecordsInRam, final Path... tmpDir) {
         if (maxRecordsInRam <= 0) {
             throw new IllegalArgumentException("maxRecordsInRam must be > 0");
         }
@@ -214,13 +215,13 @@ public class SortingCollection<T> implements Iterable<T> {
     /**
      * Sort the records in memory, write them to a file, and clear the buffer of records in memory.
      */
-    private void spillToDisk() {
+    public void spillToDisk() {
         try {
             Arrays.sort(this.ramRecords, 0, this.numRecordsInRam, this.comparator);
-            final File f = newTempFile();
+            final Path f = newTempFile();
             OutputStream os = null;
             try {
-                os = tempStreamFactory.wrapTempOutputStream(new FileOutputStream(f), Defaults.BUFFER_SIZE);
+                os = tempStreamFactory.wrapTempOutputStream(Files.newOutputStream(f), Defaults.BUFFER_SIZE);
                 this.codec.setOutputStream(os);
                 for (int i = 0; i < this.numRecordsInRam; ++i) {
                     this.codec.encode(ramRecords[i]);
@@ -230,7 +231,7 @@ public class SortingCollection<T> implements Iterable<T> {
 
                 os.flush();
             } catch (RuntimeIOException ex) {
-                throw new RuntimeIOException("Problem writing temporary file " + f.getAbsolutePath() +
+                throw new RuntimeIOException("Problem writing temporary file " + f.toUri() +
                         ".  Try setting TMP_DIR to a file system with lots of space.", ex);
             } finally {
                 if (os != null) {
@@ -251,14 +252,15 @@ public class SortingCollection<T> implements Iterable<T> {
      * Creates a new tmp file on one of the available temp filesystems, registers it for deletion
      * on JVM exit and then returns it.
      */
-    private File newTempFile() throws IOException {
-        return IOUtil.newTempFile("sortingcollection.", ".tmp", this.tmpDirs, TMP_SPACE_FREE);
+    private Path newTempFile() throws IOException {
+        return IOUtil.newTempPath("sortingcollection.", ".tmp", this.tmpDirs, TMP_SPACE_FREE);
     }
 
     /**
      * Prepare to iterate through the records in order.  This method may be called more than once,
      * but add() may not be called after this method has been called.
      */
+    @Override
     public CloseableIterator<T> iterator() {
         if (this.cleanedUp) {
             throw new IllegalStateException("Cannot call iterator() after cleanup() was called.");
@@ -280,7 +282,7 @@ public class SortingCollection<T> implements Iterable<T> {
         this.iterationStarted = true;
         this.cleanedUp = true;
 
-        IOUtil.deleteFiles(this.files);
+        IOUtil.deletePaths(this.files);
     }
 
     /**
@@ -291,13 +293,16 @@ public class SortingCollection<T> implements Iterable<T> {
      * @param comparator Defines output sort order
      * @param maxRecordsInRAM how many records to accumulate in memory before spilling to disk
      * @param tmpDir Where to write files of records that will not fit in RAM
+     *
+     * @deprecated since 2017-09. Use {@link #newInstance(Class, Codec, Comparator, int, Path...)} instead
      */
+    @Deprecated
     public static <T> SortingCollection<T> newInstance(final Class<T> componentType,
                                                        final SortingCollection.Codec<T> codec,
                                                        final Comparator<T> comparator,
                                                        final int maxRecordsInRAM,
                                                        final File... tmpDir) {
-        return new SortingCollection<T>(componentType, codec, comparator, maxRecordsInRAM, tmpDir);
+        return new SortingCollection<T>(componentType, codec, comparator, maxRecordsInRAM, Arrays.stream(tmpDir).map(File::toPath).toArray(Path[]::new));
 
     }
 
@@ -309,7 +314,10 @@ public class SortingCollection<T> implements Iterable<T> {
      * @param comparator Defines output sort order
      * @param maxRecordsInRAM how many records to accumulate in memory before spilling to disk
      * @param tmpDirs Where to write files of records that will not fit in RAM
+     *
+     * @deprecated since 2017-09. Use {@link #newInstanceFromPaths(Class, Codec, Comparator, int, Collection)} instead
      */
+    @Deprecated
     public static <T> SortingCollection<T> newInstance(final Class<T> componentType,
                                                        final SortingCollection.Codec<T> codec,
                                                        final Comparator<T> comparator,
@@ -319,7 +327,7 @@ public class SortingCollection<T> implements Iterable<T> {
                                         codec,
                                         comparator,
                                         maxRecordsInRAM,
-                                        tmpDirs.toArray(new File[tmpDirs.size()]));
+                                        tmpDirs.stream().map(File::toPath).toArray(Path[]::new));
 
     }
 
@@ -337,10 +345,50 @@ public class SortingCollection<T> implements Iterable<T> {
                                                        final Comparator<T> comparator,
                                                        final int maxRecordsInRAM) {
 
-        final File tmpDir = new File(System.getProperty("java.io.tmpdir"));
+        final Path tmpDir = Paths.get(System.getProperty("java.io.tmpdir"));
         return new SortingCollection<T>(componentType, codec, comparator, maxRecordsInRAM, tmpDir);
     }
 
+    /**
+     * Syntactic sugar around the ctor, to save some typing of type parameters
+     *
+     * @param componentType Class of the record to be sorted.  Necessary because of Java generic lameness.
+     * @param codec For writing records to file and reading them back into RAM
+     * @param comparator Defines output sort order
+     * @param maxRecordsInRAM how many records to accumulate in memory before spilling to disk
+     * @param tmpDir Where to write files of records that will not fit in RAM
+     */
+    public static <T> SortingCollection<T> newInstance(final Class<T> componentType,
+            final SortingCollection.Codec<T> codec,
+            final Comparator<T> comparator,
+            final int maxRecordsInRAM,
+            final Path... tmpDir) {
+        return new SortingCollection<T>(componentType, codec, comparator, maxRecordsInRAM, tmpDir);
+
+    }
+
+    /**
+     * Syntactic sugar around the ctor, to save some typing of type parameters
+     *
+     * @param componentType Class of the record to be sorted.  Necessary because of Java generic lameness.
+     * @param codec For writing records to file and reading them back into RAM
+     * @param comparator Defines output sort order
+     * @param maxRecordsInRAM how many records to accumulate in memory before spilling to disk
+     * @param tmpDirs Where to write files of records that will not fit in RAM
+     */
+    public static <T> SortingCollection<T> newInstanceFromPaths(final Class<T> componentType,
+            final SortingCollection.Codec<T> codec,
+            final Comparator<T> comparator,
+            final int maxRecordsInRAM,
+            final Collection<Path> tmpDirs) {
+        return new SortingCollection<T>(componentType,
+                codec,
+                comparator,
+                maxRecordsInRAM,
+                tmpDirs.toArray(new Path[tmpDirs.size()]));
+
+    }
+
     /**
      * For iteration when number of records added is less than the threshold for spilling to disk.
      */
@@ -354,14 +402,17 @@ public class SortingCollection<T> implements Iterable<T> {
                         SortingCollection.this.comparator);
         }
 
+        @Override
         public void close() {
             // nothing to do
         }
 
+        @Override
         public boolean hasNext() {
             return this.iterationIndex < SortingCollection.this.numRecordsInRam;
         }
 
+        @Override
         public T next() {
             if (!hasNext()) {
                 throw new NoSuchElementException();
@@ -372,6 +423,7 @@ public class SortingCollection<T> implements Iterable<T> {
             return ret;
         }
 
+        @Override
         public void remove() {
             throw new UnsupportedOperationException();
         }
@@ -398,7 +450,7 @@ public class SortingCollection<T> implements Iterable<T> {
         MergingIterator() {
             this.queue = new TreeSet<PeekFileRecordIterator>(new PeekFileRecordIteratorComparator());
             int n = 0;
-            for (final File f : SortingCollection.this.files) {
+            for (final Path f : SortingCollection.this.files) {
                 final FileRecordIterator it = new FileRecordIterator(f);
                 if (it.hasNext()) {
                     this.queue.add(new PeekFileRecordIterator(it, n++));
@@ -409,10 +461,12 @@ public class SortingCollection<T> implements Iterable<T> {
             }
         }
 
+        @Override
         public boolean hasNext() {
             return !this.queue.isEmpty();
         }
 
+        @Override
         public T next() {
             if (!hasNext()) {
                 throw new NoSuchElementException();
@@ -430,10 +484,12 @@ public class SortingCollection<T> implements Iterable<T> {
             return ret;
         }
 
+        @Override
         public void remove() {
             throw new UnsupportedOperationException();
         }
 
+        @Override
         public void close() {
             while (!this.queue.isEmpty()) {
                 final PeekFileRecordIterator it = this.queue.pollFirst();
@@ -446,28 +502,30 @@ public class SortingCollection<T> implements Iterable<T> {
      * Read a file of records in format defined by the codec
      */
     class FileRecordIterator implements CloseableIterator<T> {
-        private final File file;
-        private final FileInputStream is;
+        private final Path file;
+        private final InputStream is;
         private final Codec<T> codec;
         private T currentRecord = null;
 
-        FileRecordIterator(final File file) {
+        FileRecordIterator(final Path file) {
             this.file = file;
             try {
-                this.is = new FileInputStream(file);
+                this.is = Files.newInputStream(file);
                 this.codec = SortingCollection.this.codec.clone();
                 this.codec.setInputStream(tempStreamFactory.wrapTempInputStream(this.is, Defaults.BUFFER_SIZE));
                 advance();
             }
-            catch (FileNotFoundException e) {
+            catch (IOException e) {
                 throw new RuntimeIOException(e);
             }
         }
 
+        @Override
         public boolean hasNext() {
             return this.currentRecord != null;
         }
 
+        @Override
         public T next() {
             if (!hasNext()) {
                 throw new NoSuchElementException();
@@ -477,6 +535,7 @@ public class SortingCollection<T> implements Iterable<T> {
             return ret;
         }
 
+        @Override
         public void remove() {
             throw new UnsupportedOperationException();
         }
@@ -485,6 +544,7 @@ public class SortingCollection<T> implements Iterable<T> {
             this.currentRecord = this.codec.decode();
         }
 
+        @Override
         public void close() {
             CloserUtil.close(this.is);
         }
@@ -505,6 +565,7 @@ public class SortingCollection<T> implements Iterable<T> {
     class PeekFileRecordIteratorComparator implements Comparator<PeekFileRecordIterator>, Serializable {
         private static final long serialVersionUID = 1L;
 
+        @Override
         public int compare(final PeekFileRecordIterator lhs, final PeekFileRecordIterator rhs) {
             final int result = comparator.compare(lhs.peek(), rhs.peek());
             if (result == 0) return lhs.n - rhs.n;
