@@ -76,8 +76,8 @@ public class BEDCodec extends AsciiFeatureCodec<BEDFeature> {
         if (line.trim().isEmpty()) {
             return null;
         }
-
-        if (line.startsWith("#") || line.startsWith("track") || line.startsWith("browser")) {
+        // discard header lines in case the caller hasn't called readHeader
+        if (isBEDHeaderLine(line)) {
             this.readHeaderLine(line);
             return null;
         }
@@ -86,11 +86,38 @@ public class BEDCodec extends AsciiFeatureCodec<BEDFeature> {
         return decode(tokens);
     }
 
+    /**
+     * The BED codec doesn't retain the actual header, but we need to parse through
+     * it and advance to the beginning of the first feature. This is especially true
+     * if we're indexing, since we want to underlying stream offset to be established
+     * correctly, but is also the case for when we're simply iterating to satisfy a
+     * feature query (otherwise the feature reader can terminate prematurely if the
+     * header is large).
+     * @param lineIterator
+     * @return Always null. The BEDCodec currently doesn't model or preserve the BED header.
+     */
     @Override
-    public Object readActualHeader(LineIterator reader) {
+    public Object readActualHeader(final LineIterator lineIterator) {
+        while (lineIterator.hasNext()) {
+            // Only peek, since we don't want to actually consume a line of input unless its a header line.
+            // This prevents us from advancing past the first feature.
+            final String nextLine = lineIterator.peek();
+            if (isBEDHeaderLine(nextLine)) {
+                // advance the iterator and consume the line (which is a no-op)
+                this.readHeaderLine(lineIterator.next());
+            } else {
+                return null; // break out when we've seen the end of the header
+            }
+        }
+
         return null;
     }
 
+    // Return true if the candidateLine looks like a BED header line.
+    private boolean isBEDHeaderLine(final String candidateLine) {
+        return candidateLine.startsWith("#") || candidateLine.startsWith("track") || candidateLine.startsWith("browser");
+    }
+
     public BEDFeature decode(String[] tokens) {
         int tokenCount = tokens.length;
 
