@@ -28,9 +28,7 @@ import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.seekablestream.ISeekableStreamFactory;
 import htsjdk.samtools.seekablestream.SeekableStream;
 import htsjdk.samtools.seekablestream.SeekableStreamFactory;
-import htsjdk.samtools.util.BlockCompressedInputStream;
-import htsjdk.samtools.util.BlockCompressedStreamConstants;
-import htsjdk.samtools.util.LocationAware;
+import htsjdk.samtools.util.*;
 import htsjdk.tribble.*;
 import htsjdk.tribble.index.interval.IntervalIndexCreator;
 import htsjdk.tribble.index.interval.IntervalTreeIndex;
@@ -39,7 +37,7 @@ import htsjdk.tribble.index.linear.LinearIndexCreator;
 import htsjdk.tribble.index.tabix.TabixFormat;
 import htsjdk.tribble.index.tabix.TabixIndex;
 import htsjdk.tribble.index.tabix.TabixIndexCreator;
-import htsjdk.tribble.readers.PositionalBufferedStream;
+import htsjdk.tribble.readers.*;
 import htsjdk.tribble.util.LittleEndianInputStream;
 import htsjdk.tribble.util.ParsingUtils;
 import htsjdk.tribble.util.TabixUtils;
@@ -49,16 +47,20 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.EOFException;
 import java.io.InputStream;
 import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
+import java.nio.channels.SeekableByteChannel;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.function.Function;
 import java.util.zip.GZIPInputStream;
 
 /**
  * Factory class for creating indexes.  It is the responsibility of this class to determine and create the
- * correct index type from the input file or stream.  Only LinearIndex and IntervalTreeIndex are supported
+ * correct index type from the input file or stream.  LinearIndex, IntervalTreeIndex, and TabixIndex are supported
  * by this factory.
  */
 public class IndexFactory {
@@ -90,9 +92,7 @@ public class IndexFactory {
         public IndexCreator getIndexCreator() {
             try {
                 return indexCreatorClass.newInstance();
-            } catch ( final InstantiationException e ) {
-                throw new TribbleException("Couldn't make index creator in " + this, e);
-            } catch ( final IllegalAccessException e ) {
+            } catch ( final InstantiationException | IllegalAccessException e ) {
                 throw new TribbleException("Couldn't make index creator in " + this, e);
             }
         }
@@ -162,20 +162,39 @@ public class IndexFactory {
      * @param indexFile from which to load the index
      */
     public static Index loadIndex(final String indexFile) {
+        return loadIndex(indexFile, null);
+    }
+
+    /**
+     * Load in index from the specified file.   The type of index (LinearIndex or IntervalTreeIndex) is determined
+     * at run time by reading the type flag in the file.
+     *
+     * @param indexFile from which to load the index
+     * @param indexWrapper a wrapper to apply to the raw byte stream of the index file, only applied to uri's loaded as
+     *                     {@link java.nio.file.Path}
+     */
+    public static Index loadIndex(final String indexFile, Function<SeekableByteChannel, SeekableByteChannel> indexWrapper) {
         // Must be buffered, because getIndexType uses mark and reset
-        try (BufferedInputStream bufferedInputStream = new BufferedInputStream(indexFileInputStream(indexFile), Defaults.NON_ZERO_BUFFER_SIZE)){
+        try (BufferedInputStream bufferedInputStream = new BufferedInputStream(indexFileInputStream(indexFile, indexWrapper), Defaults.NON_ZERO_BUFFER_SIZE)) {
             final Class<Index> indexClass = IndexType.getIndexType(bufferedInputStream).getIndexType();
             final Constructor<Index> ctor = indexClass.getConstructor(InputStream.class);
             return ctor.newInstance(bufferedInputStream);
+        } catch (final TribbleException ex) {
+            throw ex;
         } catch (final IOException ex) {
             throw new TribbleException.UnableToReadIndexFile("Unable to read index file", indexFile, ex);
+        } catch (final InvocationTargetException ex) {
+            if (ex.getCause() instanceof EOFException) {
+                throw new TribbleException.CorruptedIndexFile("Index file is corrupted", indexFile, ex);
+            }
+            throw new RuntimeException(ex);
         } catch (final Exception ex) {
             throw new RuntimeException(ex);
         }
     }
 
-    private static InputStream indexFileInputStream(final String indexFile) throws IOException {
-        final InputStream inputStreamInitial = ParsingUtils.openInputStream(indexFile);
+    private static InputStream indexFileInputStream(final String indexFile, Function<SeekableByteChannel, SeekableByteChannel> indexWrapper) throws IOException {
+        final InputStream inputStreamInitial = ParsingUtils.openInputStream(indexFile, indexWrapper);
         if (indexFile.endsWith(".gz")) {
             return new GZIPInputStream(inputStreamInitial);
         }
@@ -364,8 +383,10 @@ public class IndexFactory {
             lastFeature = currentFeature;
         }
 
+        // Get the end position of the last feature before closing the iterator
+        long finalPosition = iterator.getPosition();
         iterator.close();
-        return creator.finalizeIndex(iterator.getPosition());
+        return creator.finalizeIndex(finalPosition);
     }
 
     private static String featToString(final Feature feature){
@@ -398,62 +419,53 @@ public class IndexFactory {
 
         /**
          *
-         * @param inputFile The file from which to read. Stream for reading is opened on construction.
+         * @param inputFile The file from which to read. Stream for reading is opened on construction. May not be null.
          * @param codec
          */
         public FeatureIterator(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE> codec) {
+            if (inputFile == null) {
+                throw new IllegalArgumentException("FeatureIterator input file cannot be null");
+            }
             this.codec = codec;
             this.inputFile = inputFile;
-            final FeatureCodecHeader header = readHeader();
-            source = (SOURCE) codec.makeIndexableSourceFromStream(initStream(inputFile, header.getHeaderEnd()));
-            readNextFeature();
-        }
-
-        /**
-         * Some codecs,  e.g. VCF files,  need the header to decode features.  This is a rather poor design,
-         * the internal header is set as a side-affect of reading it, but we have to live with it for now.
-         */
-        private FeatureCodecHeader readHeader() {
             try {
-                final SOURCE source = this.codec.makeSourceFromStream(initStream(inputFile, 0));
-                final FeatureCodecHeader header = this.codec.readHeader(source);
-                codec.close(source);
-                return header;
+                if (AbstractFeatureReader.hasBlockCompressedExtension(inputFile)) {
+                    final BlockCompressedInputStream bcs = initIndexableBlockCompressedStream(inputFile);
+                    source = (SOURCE) codec.makeIndexableSourceFromStream(bcs);
+                } else {
+                    final PositionalBufferedStream ps = initIndexablePositionalStream(inputFile);
+                    source = (SOURCE) codec.makeIndexableSourceFromStream(ps);
+                }
+                this.codec.readHeader(source);
+                readNextFeature();
             } catch (final IOException e) {
                 throw new TribbleException.InvalidHeader("Error reading header " + e.getMessage());
             }
         }
 
-        private PositionalBufferedStream initStream(final File inputFile, final long skip) {
+        private static PositionalBufferedStream initIndexablePositionalStream(final File inputFile) {
             try {
                 final FileInputStream fileStream = new FileInputStream(inputFile);
-                final InputStream is;
+                return new PositionalBufferedStream(fileStream);
+            } catch (final FileNotFoundException e) {
+                throw new TribbleException.FeatureFileDoesntExist("Unable to open the input file, most likely the file doesn't exist.", inputFile.getAbsolutePath());
+            }
+        }
 
-                // if this looks like a block compressed file and it in fact is, we will use it
-                // otherwise we will use the file as is
-                if (AbstractFeatureReader.hasBlockCompressedExtension(inputFile)) {
-                    // make a buffered stream to test that this is in fact a valid block compressed file
-                    final int bufferSize = Math.max(Defaults.BUFFER_SIZE, BlockCompressedStreamConstants.MAX_COMPRESSED_BLOCK_SIZE);
-                    final BufferedInputStream bufferedStream = new BufferedInputStream(fileStream, bufferSize);
+        private static BlockCompressedInputStream initIndexableBlockCompressedStream(final File inputFile) {
+            final int bufferSize = Math.max(Defaults.BUFFER_SIZE, BlockCompressedStreamConstants.MAX_COMPRESSED_BLOCK_SIZE);
 
-                    if (!BlockCompressedInputStream.isValidFile(bufferedStream)) {
-                        throw new TribbleException.MalformedFeatureFile("Input file is not in valid block compressed format.", inputFile.getAbsolutePath());
-                    }
+            // make a buffered stream to test that this is in fact a valid block compressed file
+            try (final BufferedInputStream bufferedStream = new BufferedInputStream(new FileInputStream(inputFile), bufferSize)){
 
-                    final ISeekableStreamFactory ssf = SeekableStreamFactory.getInstance();
-                    // if we got here, the file is valid, make a SeekableStream for the BlockCompressedInputStream to read from
-                    final SeekableStream seekableStream =
-                            ssf.getBufferedStream(ssf.getStreamFor(inputFile.getAbsolutePath()));
-                    is = new BlockCompressedInputStream(seekableStream);
-                }
-                else {
-                    is = fileStream;
+                if (!BlockCompressedInputStream.isValidFile(bufferedStream)) {
+                    throw new TribbleException.MalformedFeatureFile("Input file is not in valid block compressed format.",
+                            inputFile.getAbsolutePath());
                 }
 
-                final PositionalBufferedStream pbs = new PositionalBufferedStream(is);
-
-                if ( skip > 0 ) pbs.skip(skip);
-                return pbs;
+                final ISeekableStreamFactory ssf = SeekableStreamFactory.getInstance();
+                final SeekableStream seekableStream = ssf.getStreamFor(inputFile.getAbsolutePath());
+                return new BlockCompressedInputStream(seekableStream);
             } catch (final FileNotFoundException e) {
                 throw new TribbleException.FeatureFileDoesntExist("Unable to open the input file, most likely the file doesn't exist.", inputFile.getAbsolutePath());
             } catch (final IOException e) {
@@ -461,10 +473,12 @@ public class IndexFactory {
             }
         }
 
+        @Override
         public boolean hasNext() {
             return nextFeature != null;
         }
 
+        @Override
         public Feature next() {
             final Feature ret = nextFeature;
             readNextFeature();
@@ -474,6 +488,7 @@ public class IndexFactory {
         /**
          * @throws UnsupportedOperationException
          */
+        @Override
         public void remove() {
             throw new UnsupportedOperationException("We cannot remove");
         }
