@@ -53,7 +53,6 @@ import java.util.EnumSet;
  * 
  * <p>
  * Provides methods for creating <code>VariantContextWriter</code>s using the Builder pattern.
- * Replaces <code>VariantContextWriterFactory</code>.
  * </p>
  * <p>
  * The caller must choose an output file or an output stream for the <code>VariantContextWriter</code> to write to.
@@ -407,8 +406,9 @@ public class VariantContextWriterBuilder {
                 typeToBuild = OutputType.BCF_STREAM;
         }
 
+        // If we are writing to a file, or a special file type (ex. pipe) where the stream is not yet open.
         OutputStream outStreamFromFile = this.outStream;
-        if (FILE_TYPES.contains(this.outType)) {
+        if (FILE_TYPES.contains(this.outType) || (STREAM_TYPES.contains(this.outType) && this.outStream == null)) {
             try {
                 outStreamFromFile = IOUtil.maybeBufferOutputStream(new FileOutputStream(outFile), bufferSize);
             } catch (final FileNotFoundException e) {
@@ -446,7 +446,7 @@ public class VariantContextWriterBuilder {
                 if (options.contains(Options.INDEX_ON_THE_FLY))
                     throw new IllegalArgumentException("VCF index creation not supported for stream output.");
 
-                writer = createVCFWriter(null, outStream);
+                writer = createVCFWriter(null, outStreamFromFile);
                 break;
             case BCF_STREAM:
                 if (options.contains(Options.INDEX_ON_THE_FLY))
@@ -491,11 +491,11 @@ public class VariantContextWriterBuilder {
     }
 
     private static boolean isVCF(final File outFile) {
-        return outFile != null && outFile.getName().endsWith(".vcf");
+        return outFile != null && outFile.getName().endsWith(IOUtil.VCF_FILE_EXTENSION);
     }
 
     private static boolean isBCF(final File outFile) {
-        return outFile != null && outFile.getName().endsWith(".bcf");
+        return outFile != null && outFile.getName().endsWith(IOUtil.BCF_FILE_EXTENSION);
     }
 
     private static boolean isCompressedVCF(final File outFile) {
