@@ -1,11 +1,13 @@
 package htsjdk.samtools;
 
+import htsjdk.HtsjdkTest;
 import htsjdk.samtools.cram.ref.ReferenceSource;
 import htsjdk.samtools.seekablestream.ISeekableStreamFactory;
 import htsjdk.samtools.seekablestream.SeekableFileStream;
 import htsjdk.samtools.seekablestream.SeekableHTTPStream;
 import htsjdk.samtools.seekablestream.SeekableStreamFactory;
 import htsjdk.samtools.util.*;
+import htsjdk.samtools.util.zip.InflaterFactory;
 import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
@@ -16,14 +18,20 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.nio.ByteBuffer;
+import java.nio.channels.SeekableByteChannel;
 import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.concurrent.*;
 import java.util.function.BiFunction;
+import java.util.function.Function;
+import java.util.zip.Inflater;
 
-public class SamReaderFactoryTest {
+public class SamReaderFactoryTest extends HtsjdkTest {
     private static final File TEST_DATA_DIR = new File("src/test/resources/htsjdk/samtools");
 
     private static final Log LOG = Log.getInstance(SamReaderFactoryTest.class);
@@ -37,6 +45,34 @@ public class SamReaderFactoryTest {
         reader.close();
     }
 
+    @Test
+    public void variousFormatReaderInflatorFactoryTest() throws IOException {
+        final String inputFile = "compressed.bam";
+        final int[] inflateCalls = {0}; //Note: using an array is a HACK to fool the compiler
+        class MyInflater extends Inflater {
+            MyInflater(boolean gzipCompatible){
+                super(gzipCompatible);
+            }
+            @Override
+            public int inflate(byte[] b, int off, int len) throws java.util.zip.DataFormatException {
+                inflateCalls[0]++;
+                return super.inflate(b, off, len);
+            }
+        }
+        final InflaterFactory myInflaterFactory = new InflaterFactory() {
+            @Override
+            public Inflater makeInflater(final boolean gzipCompatible) {
+                return new MyInflater(gzipCompatible);
+            }
+        };
+
+        final File input = new File(TEST_DATA_DIR, inputFile);
+        try (final SamReader reader = SamReaderFactory.makeDefault().inflaterFactory(myInflaterFactory).open(input)) {
+            for (final SAMRecord ignored : reader) { }
+        }
+        Assert.assertNotEquals(inflateCalls[0], 0, "Not using Inflater from InflateFactory on file : " + inputFile);
+    }
+
     private int countRecordsInQueryInterval(final SamReader reader, final QueryInterval query) {
         final SAMRecordIterator iter = reader.queryOverlapping(new QueryInterval[] { query });
         int count = 0;
@@ -59,6 +95,40 @@ public class SamReaderFactoryTest {
         return count;
     }
 
+    private static SeekableByteChannel addHeader(SeekableByteChannel input) {
+        try {
+        int total = (int)input.size();
+        final String comment = "@HD\tVN:1.0  SO:unsorted\n" +
+            "@SQ\tSN:chr1\tLN:101\n" +
+            "@SQ\tSN:chr2\tLN:101\n" +
+            "@SQ\tSN:chr3\tLN:101\n" +
+            "@RG\tID:0\tSM:JP was here\n";
+
+            byte[] commentBuf = comment.getBytes();
+        ByteBuffer buf = ByteBuffer.allocate(total + commentBuf.length);
+        buf.put(commentBuf);
+        input.position(0);
+        while (input.read(buf)>0) {
+            // read until EOF
+        }
+        buf.flip();
+        return new SeekableByteChannelFromBuffer(buf);
+        } catch (IOException x) {
+            throw new RuntimeException(x);
+        }
+    }
+
+    @Test
+    public void testWrap() throws IOException {
+        final Path input = Paths.get(TEST_DATA_DIR.getPath(), "noheader.sam");
+        final SamReader wrappedReader =
+            SamReaderFactory
+                .makeDefault()
+                .open(input, SamReaderFactoryTest::addHeader, null);
+        int records = countRecords(wrappedReader);
+        Assert.assertEquals(10, records);
+    }
+
     // See https://github.com/samtools/htsjdk/issues/76
     @Test(dataProvider = "queryIntervalIssue76TestCases")
     public void queryIntervalIssue76(final String sequenceName, final int start, final int end, final int expectedCount) throws IOException {
@@ -94,11 +164,13 @@ public class SamReaderFactoryTest {
         int samRecordsCreated;
         int bamRecordsCreated;
 
+        @Override
         public SAMRecord createSAMRecord(final SAMFileHeader header) {
             ++samRecordsCreated;
             return super.createSAMRecord(header);
         }
 
+        @Override
         public BAMRecord createBAMRecord(final SAMFileHeader header, final int referenceSequenceIndex, final int alignmentStart, final short readNameLength, final short mappingQuality, final int indexingBin, final int cigarLen, final int flags, final int readLen, final int mateReferenceSequenceIndex, final int mateAlignmentStart, final int insertSize, final byte[] variableLengthBlock) {
             ++bamRecordsCreated;
             return super.createBAMRecord(header, referenceSequenceIndex, alignmentStart, readNameLength, mappingQuality, indexingBin, cigarLen, flags, readLen, mateReferenceSequenceIndex, mateAlignmentStart, insertSize, variableLengthBlock);
@@ -194,7 +266,7 @@ public class SamReaderFactoryTest {
             case FILE:
                 return new FileInputResource(f);
             case PATH:
-                return new PathInputResource(f.toPath());
+                return new PathInputResource(f.toPath(), Function.identity());
             case URL:
                 return new UrlInputResource(url);
             case SEEKABLE_STREAM:
@@ -250,7 +322,6 @@ public class SamReaderFactoryTest {
         }
     }
 
-
     final Set<List<SAMRecord>> observedRecordOrdering1 = new HashSet<List<SAMRecord>>();
     final Set<List<SAMRecord>> observedRecordOrdering3 = new HashSet<List<SAMRecord>>();
     final Set<List<SAMRecord>> observedRecordOrdering20 = new HashSet<List<SAMRecord>>();
@@ -465,4 +536,57 @@ public class SamReaderFactoryTest {
         countRecords(reader);
     }
 
+    @Test(singleThreaded = true, groups="unix")
+    public void testWriteAndReadFromPipe() throws IOException, InterruptedException, ExecutionException, TimeoutException {
+        final File fifo = File.createTempFile("fifo", "");
+        Assert.assertTrue(fifo.delete());
+        fifo.deleteOnExit();
+        final Process exec = Runtime.getRuntime().exec(new String[]{"mkfifo", fifo.getAbsolutePath()});
+        exec.waitFor(1, TimeUnit.MINUTES);
+        Assert.assertEquals(exec.exitValue(), 0, "mkfifo failed with exit code " + 0);
+
+        ExecutorService executor = null;
+        try {
+            executor = Executors.newFixedThreadPool(2);
+            final File input = new File(TEST_DATA_DIR, "example.bam");
+            final Future<Integer> writing = executor.submit(writeToPipe(fifo, input));
+            final Future<Integer> reading = executor.submit(readFromPipe(fifo));
+            Assert.assertEquals(writing.get(1, TimeUnit.MINUTES), reading.get(1, TimeUnit.MINUTES));
+        } finally {
+            if (executor != null) {
+                executor.shutdownNow();
+            }
+        }
+    }
+
+    private static Callable<Integer> readFromPipe(File fifo) {
+        return () -> {
+            try (final SamReader reader = SamReaderFactory.makeDefault().open(fifo)) {
+                return (int)reader.iterator().stream().count();
+            } catch (Exception e) {
+                Assert.fail("failed during reading from pipe", e);
+            }
+            throw new RuntimeException("Shouldn't actually reach here but the compiler was confused");
+        };
+    }
+
+    private static Callable<Integer> writeToPipe(File fifo, File input) {
+        return () -> {
+            int written = 0;
+            try {
+                try (final SamReader reader = SamReaderFactory.makeDefault().open(input);
+                     final SAMFileWriter writer = new SAMFileWriterFactory().setCreateIndex(false)
+                             .setCreateMd5File(false)
+                             .makeBAMWriter(reader.getFileHeader(), true, fifo)) {
+                    for (SAMRecord read : reader) {
+                        writer.addAlignment(read);
+                        written++;
+                    }
+                }
+            } catch (final Exception e) {
+                Assert.fail("Failed during writing to pipe", e);
+            }
+            return written;
+        };
+    }
 }
