@@ -24,19 +24,26 @@
 
 package htsjdk.samtools.reference;
 
+import htsjdk.HtsjdkTest;
 import htsjdk.samtools.SAMException;
 import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
+import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileNotFoundException;
+import java.io.FileReader;
+import java.nio.file.Files;
 import java.util.Iterator;
+import java.util.List;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 /**
  * Test the fasta sequence index reader.
  */
-public class FastaSequenceIndexTest {
+public class FastaSequenceIndexTest extends HtsjdkTest {
     private static File TEST_DATA_DIR = new File("src/test/resources/htsjdk/samtools/reference");
 
     @DataProvider(name="homosapiens")
@@ -253,4 +260,30 @@ public class FastaSequenceIndexTest {
         Assert.assertEquals(ent.getBasesPerLine(),70,"Contig file:gi|17981852|ref|NC_001807.4| bases per line is not correct");
         Assert.assertEquals(ent.getBytesPerLine(),71,"Contig file:gi|17981852|ref|NC_001807.4| bytes per line is not correct");
     }
+
+    @Test
+    public void testWrite() throws Exception {
+        // gets the original file and index
+        final File originalFile = new File(TEST_DATA_DIR, "testing.fai");
+        final FastaSequenceIndex originalIndex = new FastaSequenceIndex(originalFile);
+
+        // write the index to a temp file and test if files are the same
+        final File fileToWrite = File.createTempFile("testing.toWrite", "fai");
+        fileToWrite.deleteOnExit();
+        originalIndex.write(fileToWrite.toPath());
+
+        // read all the files and compare line by line
+        try(final Stream<String> original = Files.lines(originalFile.toPath());
+            final Stream<String> written = Files.lines(fileToWrite.toPath())) {
+            final List<String> originalLines = original.filter(s -> ! s.isEmpty()).collect(Collectors.toList());
+            final List<String> actualLines = written.filter(s -> !s.isEmpty()).collect(Collectors.toList());
+            Assert.assertEquals(actualLines, originalLines);
+        }
+
+        // load the tmp index and check that both are the same
+        final FastaSequenceIndex writtenIndex = new FastaSequenceIndex(fileToWrite);
+        Assert.assertEquals(writtenIndex, originalIndex);
+    }
+
+
 }
