@@ -24,8 +24,8 @@
 
 package htsjdk.samtools.util;
 
-import htsjdk.samtools.SAMFileHeader;
-import htsjdk.samtools.SAMSequenceRecord;
+import htsjdk.HtsjdkTest;
+import htsjdk.samtools.*;
 import htsjdk.variant.vcf.VCFFileReader;
 import org.testng.Assert;
 import org.testng.annotations.BeforeTest;
@@ -33,6 +33,7 @@ import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 import java.io.File;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -45,7 +46,7 @@ import java.util.TreeSet;
 /**
  * Tests the IntervalList class
  */
-public class IntervalListTest {
+public class IntervalListTest extends HtsjdkTest {
 
     final SAMFileHeader fileHeader;
     final IntervalList list1, list2, list3;
@@ -75,6 +76,15 @@ public class IntervalListTest {
         list3.add(new Interval("3", 50, 470));
     }
 
+    @Test
+    public void testIntervalListFrom() {
+        final String testPath = "src/test/resources/htsjdk/samtools/intervallist/IntervalListFromVCFTestComp.interval_list";
+        final IntervalList fromFileList = IntervalList.fromFile(new File(testPath));
+        final IntervalList fromPathList = IntervalList.fromPath(Paths.get(testPath));
+        fromFileList.getHeader().getSequenceDictionary().assertSameDictionary(fromPathList.getHeader().getSequenceDictionary());
+        Assert.assertEquals(CollectionUtil.makeCollection(fromFileList.iterator()), CollectionUtil.makeCollection(fromPathList.iterator()));
+    }
+
     @DataProvider(name = "intersectData")
     public Object[][] intersectData() {
         final IntervalList intersect123 = new IntervalList(fileHeader);
@@ -364,12 +374,97 @@ public class IntervalListTest {
     }
 
     @Test(dataProvider = "subtractSingletonData")
-    public void testSubtractSingletonasListIntervalList(final IntervalList fromLists, final IntervalList whatLists, final IntervalList list) {
+    public void testSubtractSingletonAsListIntervalList(final IntervalList fromLists, final IntervalList whatLists, final IntervalList list) {
         Assert.assertEquals(
                 CollectionUtil.makeCollection(IntervalList.subtract(Collections.singletonList(fromLists), Collections.singletonList(whatLists)).iterator()),
                 CollectionUtil.makeCollection(list.iterator()));
     }
 
+    @DataProvider(name = "overlapsSingletonData")
+    public Object[][] overlapSingletonData() {
+        final IntervalList two_overlaps_one   = new IntervalList(fileHeader);
+        final IntervalList three_overlaps_two = new IntervalList(fileHeader);
+        final IntervalList three_overlaps_one = new IntervalList(fileHeader);
+        final IntervalList one_overlaps_three = new IntervalList(fileHeader);
+
+        // NB: commented lines below are there to show the intervals in the first list that will not be in the resulting list
+
+        two_overlaps_one.add(new Interval("1", 50, 150));
+        //two_overlaps_one.add(new Interval("1", 301, 500));
+        two_overlaps_one.add(new Interval("2", 1, 150));
+        two_overlaps_one.add(new Interval("2", 250, 270));
+        two_overlaps_one.add(new Interval("2", 290, 400));
+
+        three_overlaps_two.add(new Interval("1", 25, 400));
+        three_overlaps_two.add(new Interval("2", 200, 600));
+        //three_overlaps_two.add(new Interval("3", 50, 470));
+
+        three_overlaps_one.add(new Interval("1", 25, 400));
+        three_overlaps_one.add(new Interval("2", 200, 600));
+        //three_overlaps_one.add(new Interval("3", 50, 470));
+
+        one_overlaps_three.add(new Interval("1", 1, 100));
+        one_overlaps_three.add(new Interval("1", 101, 200));
+        one_overlaps_three.add(new Interval("1", 202, 300));
+        one_overlaps_three.add(new Interval("2", 200, 300));
+        //one_overlaps_three.add(new Interval("2", 100, 150));
+
+        return new Object[][]{
+                new Object[]{list1, list1, list1}, // should return itself
+                new Object[]{list1, IntervalList.invert(list1), new IntervalList(list1.getHeader())}, // should be empty
+                new Object[]{list2, list1, two_overlaps_one},
+                new Object[]{list3, list2, three_overlaps_two},
+                new Object[]{list3, list1, three_overlaps_one},
+                new Object[]{list1, list3, one_overlaps_three}
+        };
+    }
+
+    @DataProvider(name = "overlapsData")
+    public Object[][] overlapData() {
+        final IntervalList three_overlaps_one_and_two = new IntervalList(fileHeader);
+
+        three_overlaps_one_and_two.add(new Interval("1", 25, 400));
+        three_overlaps_one_and_two.add(new Interval("2", 200, 600));
+        //three_overlaps_one_and_two.add(new Interval("3", 50, 470));
+
+        return new Object[][]{
+                new Object[]{CollectionUtil.makeList(list3), CollectionUtil.makeList(list1, list2), three_overlaps_one_and_two},
+        };
+    }
+
+    @Test(dataProvider = "overlapsData")
+    public void testOverlapsIntervalLists(final List<IntervalList> fromLists, final List<IntervalList> whatLists, final IntervalList list) {
+        Assert.assertEquals(
+                CollectionUtil.makeCollection(IntervalList.overlaps(fromLists, whatLists).iterator()),
+                CollectionUtil.makeCollection(list.iterator()));
+    }
+
+    @Test(dataProvider = "overlapsSingletonData")
+    public void testOverlapsSingletonIntervalLists(final IntervalList fromLists, final IntervalList whatLists, final IntervalList list) {
+        Assert.assertEquals(
+                CollectionUtil.makeCollection(IntervalList.overlaps(fromLists, whatLists).iterator()),
+                CollectionUtil.makeCollection(list.iterator()));
+    }
+
+    @Test(dataProvider = "overlapsSingletonData")
+    public void testOverlapsSingletonAsListIntervalList(final IntervalList fromLists, final IntervalList whatLists, final IntervalList list) {
+        Assert.assertEquals(
+                CollectionUtil.makeCollection(IntervalList.overlaps(Collections.singletonList(fromLists), Collections.singletonList(whatLists)).iterator()),
+                CollectionUtil.makeCollection(list.iterator()));
+    }
+
+    @Test(expectedExceptions = SAMException.class)
+    public void testOverlapsEmptyFirstList() {
+        IntervalList.overlaps(Collections.emptyList(), Collections.singletonList(list1));
+    }
+
+    @Test
+    public void testOverlapsEmptySecondList() {
+        Assert.assertEquals(
+                CollectionUtil.makeCollection(IntervalList.overlaps(Collections.singletonList(list1), Collections.emptyList()).iterator()),
+                Collections.emptyList());
+    }
+
     @DataProvider(name = "VCFCompData")
     public Object[][] VCFCompData() {
         return new Object[][]{
@@ -410,6 +505,38 @@ public class IntervalListTest {
         Assert.assertEquals(intervalNames, compIntervalNames);
     }
 
+
+    @Test(dataProvider = "VCFCompData")
+    public void testFromVCFWithPath(final String vcf, final String compInterval, final boolean invertVCF) {
+
+        final File vcfFile = new File(vcf);
+        final File compIntervalFile = new File(compInterval);
+
+        final IntervalList compList = IntervalList.fromFile(compIntervalFile);
+        final IntervalList list = invertVCF ? IntervalList.invert(VCFFileReader.fromVcf(vcfFile)) : VCFFileReader.fromVcf(vcfFile);
+
+        compList.getHeader().getSequenceDictionary().assertSameDictionary(list.getHeader().getSequenceDictionary());
+
+        final Collection<Interval> intervals = CollectionUtil.makeCollection(list.iterator());
+        final Collection<Interval> compIntervals = CollectionUtil.makeCollection(compList.iterator());
+
+        //assert that the intervals correspond
+        Assert.assertEquals(intervals, compIntervals);
+
+        final List<String> intervalNames = new LinkedList<String>();
+        final List<String> compIntervalNames = new LinkedList<String>();
+
+        for (final Interval interval : intervals) {
+            intervalNames.add(interval.getName());
+        }
+        for (final Interval interval : compIntervals) {
+            compIntervalNames.add(interval.getName());
+        }
+        //assert that the names match
+        Assert.assertEquals(intervalNames, compIntervalNames);
+    }
+
+
     @DataProvider
     public Object[][] testFromSequenceData() {
         return new Object[][]{
@@ -517,4 +644,23 @@ public class IntervalListTest {
         Assert.assertTrue(false);
 
     }
+
+    @Test public void uniqueIntervalsWithoutNames() {
+        final IntervalList test = new IntervalList(this.fileHeader);
+        test.add(new Interval("1", 100, 200));
+        test.add(new Interval("1", 500, 600));
+        test.add(new Interval("1", 550, 700));
+
+        for (final boolean concat : new boolean[]{true, false}) {
+            final IntervalList unique = test.uniqued(concat);
+            Assert.assertEquals(unique.size(), 2);
+        }
+    }
+
+    @Test(expectedExceptions = IllegalArgumentException.class)
+    public void testContigsAbsentInHeader() {
+        String vcf = "src/test/resources/htsjdk/samtools/intervallist/IntervalListFromVCFNoContigLines.vcf";
+        final File vcfFile = new File(vcf);
+        VCFFileReader.fromVcf(vcfFile);
+    }
 }
